@@ -102,7 +102,7 @@ int main(int argc,char *argv[])
   MY_INIT(argv[0]);
   sf_leaking_memory=1; /* don't report memory leaks on early exits */
   plugin_data.name= 0; /* initialize name                          */
-  
+
   /*
     The following operations comprise the method for enabling or disabling
     a plugin. We begin by processing the command options then check the
@@ -110,15 +110,15 @@ int main(int argc,char *argv[])
     --plugin-ini (if specified). If the directories are Ok, we then look
     for the mysqld executable and the plugin soname. Finally, we build a
     bootstrap command file for use in bootstraping the server.
-    
+
     If any step fails, the method issues an error message and the tool exits.
-    
+
       1) Parse, execute, and verify command options.
       2) Check access to directories.
       3) Look for mysqld executable.
       4) Look for the plugin.
       5) Build a bootstrap file with commands to enable or disable plugin.
-      
+
   */
   if ((error= process_options(argc, argv, operation)) ||
       (error= check_access()) ||
@@ -126,11 +126,11 @@ int main(int argc,char *argv[])
       (error= find_plugin(tp_path)) ||
       (error= build_bootstrap_file(operation, bootstrap)))
     goto exit;
-  
+
   /* Dump the bootstrap file if --verbose specified. */
   if (opt_verbose && ((error= dump_bootstrap_file(bootstrap))))
     goto exit;
-  
+
   /* Start the server in bootstrap mode and execute bootstrap commands */
   error= bootstrap_server(server_path, bootstrap);
 
@@ -238,7 +238,7 @@ static int run_command(char* cmd, const char *mode)
 #ifdef _WIN32
 /**
   Check to see if there are spaces in a path.
-  
+
   @param[in]  path  The Windows path to examine.
 
   @retval int spaces found = 1, no spaces = 0
@@ -253,7 +253,7 @@ static int has_spaces(const char *path)
 
 /**
   Convert a Unix path to a Windows path.
- 
+
   @param[in]  path  The Windows path to examine.
 
   @returns string containing path with / changed to \\
@@ -281,7 +281,7 @@ static char *convert_path(const char *argument)
 
   @param[in]  path  The Windows path to examine.
 
-  @returns string containing excaped quotes if spaces found in path
+  @returns string containing escaped quotes if spaces found in path
 */
 static char *add_quotes(const char *path)
 {
@@ -335,12 +335,12 @@ static int get_default_values()
 #ifdef _WIN32
     {
       char *format_str= 0;
-  
+
       if (has_spaces(tool_path) || has_spaces(defaults_file))
         format_str = "\"%s --mysqld > %s\"";
       else
         format_str = "%s --mysqld > %s";
-  
+
       snprintf(defaults_cmd, sizeof(defaults_cmd), format_str,
                add_quotes(tool_path), add_quotes(defaults_file));
       if (opt_verbose)
@@ -569,14 +569,14 @@ static int file_exists(char * filename)
   @retval int error = 1, success = 0
 */
 
-static int search_dir(const char * base_path, const char *tool_name,
+static int search_dir(const char *base_path, const char *tool_name,
                       const char *subdir, char *tool_path)
 {
   char new_path[FN_REFLEN];
   char source_path[FN_REFLEN];
 
-  strcpy(source_path, base_path);
-  strcat(source_path, subdir);
+  safe_strcpy(source_path, sizeof(source_path), base_path);
+  safe_strcat(source_path, sizeof(source_path), subdir);
   fn_format(new_path, tool_name, source_path, "", MY_UNPACK_FILENAME);
   if (file_exists(new_path))
   {
@@ -632,7 +632,7 @@ static int load_plugin_data(char *plugin_name, char *config_file)
   FILE *file_ptr;
   char path[FN_REFLEN];
   char line[1024];
-  char *reason= 0;
+  const char *reason= 0;
   char *res;
   int i= -1;
 
@@ -643,14 +643,14 @@ static int load_plugin_data(char *plugin_name, char *config_file)
   }
   if (!file_exists(opt_plugin_ini))
   {
-    reason= (char *)"File does not exist.";
+    reason= "File does not exist.";
     goto error;
   }
 
   file_ptr= fopen(opt_plugin_ini, "r");
   if (file_ptr == NULL)
   {
-    reason= (char *)"Cannot open file.";
+    reason= "Cannot open file.";
     goto error;
   }
 
@@ -660,19 +660,22 @@ static int load_plugin_data(char *plugin_name, char *config_file)
   /* Read plugin components */
   while (i < 16)
   {
+    size_t line_len;
+
     res= fgets(line, sizeof(line), file_ptr);
+    line_len= strlen(line);
+
     /* strip /n */
-    if (line[strlen(line)-1] == '\n')
-    {
-      line[strlen(line)-1]= '\0';
-    }
+    if (line[line_len - 1] == '\n')
+      line[line_len - 1]= '\0';
+
     if (res == NULL)
     {
       if (i < 1)
       {
-        reason= (char *)"Bad format in plugin configuration file.";
+        reason= "Bad format in plugin configuration file.";
         fclose(file_ptr);
-        goto error;        
+        goto error;
       }
       break;
     }
@@ -683,14 +686,19 @@ static int load_plugin_data(char *plugin_name, char *config_file)
     if (i == -1) /* if first pass, read this line as so_name */
     {
       /* Add proper file extension for soname */
-      strcat(line, FN_SOEXT);
+      if (safe_strcpy_truncated(line + line_len - 1, sizeof line, FN_SOEXT))
+      {
+        reason= "Plugin name too long.";
+        fclose(file_ptr);
+        goto error;
+      }
       /* save so_name */
       plugin_data.so_name= my_strdup(PSI_NOT_INSTRUMENTED, line, MYF(MY_WME|MY_ZEROFILL));
       i++;
     }
     else
     {
-      if (strlen(line) > 0)
+      if (line_len > 0)
       {
         plugin_data.components[i]= my_strdup(PSI_NOT_INSTRUMENTED, line, MYF(MY_WME));
         i++;
@@ -701,7 +709,7 @@ static int load_plugin_data(char *plugin_name, char *config_file)
       }
     }
   }
-  
+
   fclose(file_ptr);
   return 0;
 
@@ -730,9 +738,9 @@ static int check_options(int argc, char **argv, char *operation)
 {
   int i= 0;                    /* loop counter */
   int num_found= 0;            /* number of options found (shortcut loop) */
-  char config_file[FN_REFLEN]; /* configuration file name */
-  char plugin_name[FN_REFLEN]; /* plugin name */
-  
+  char config_file[FN_REFLEN+1]; /* configuration file name */
+  char plugin_name[FN_REFLEN+1]; /* plugin name */
+
   /* Form prefix strings for the options. */
   const char *basedir_prefix = "--basedir=";
   size_t basedir_len= strlen(basedir_prefix);
@@ -741,7 +749,7 @@ static int check_options(int argc, char **argv, char *operation)
   const char *plugin_dir_prefix = "--plugin_dir=";
   size_t plugin_dir_len= strlen(plugin_dir_prefix);
 
-  strcpy(plugin_name, "");
+  *plugin_name= '\0';
   for (i = 0; i < argc && num_found < 5; i++)
   {
 
@@ -779,14 +787,13 @@ static int check_options(int argc, char **argv, char *operation)
     /* read the plugin config file and check for match against argument */
     else
     {
-      if (strlen(argv[i]) + 4 + 1 > FN_REFLEN)
+      if (safe_strcpy_truncated(plugin_name, sizeof(plugin_name)-1, argv[i]) ||
+          safe_strcpy_truncated(config_file, sizeof(config_file)-1, argv[i]) ||
+          safe_strcat(config_file, sizeof(config_file), ".ini"))
       {
         fprintf(stderr, "ERROR: argument is too long.\n");
         return 1;
       }
-      strcpy(plugin_name, argv[i]);
-      strcpy(config_file, argv[i]);
-      strcat(config_file, ".ini");
     }
   }
 
@@ -808,7 +815,7 @@ static int check_options(int argc, char **argv, char *operation)
     return 1;
   }
   /* If a plugin was specified, read the config file. */
-  else if (strlen(plugin_name) > 0) 
+  else if (strlen(plugin_name) > 0)
   {
     if (load_plugin_data(plugin_name, config_file))
     {
@@ -840,65 +847,57 @@ static int check_options(int argc, char **argv, char *operation)
 
 /**
   Parse, execute, and verify command options.
-  
+
   This method handles all of the option processing including the optional
   features for displaying data (--print-defaults, --help ,etc.) that do not
   result in an attempt to ENABLE or DISABLE of a plugin.
-  
+
   @param[in]   arc        Count of arguments
   @param[in]   argv       Array of arguments
   @param[out]  operation  Operation (ENABLE or DISABLE)
-  
+
   @retval int error = 1, success = 0, exit program = -1
 */
 
 static int process_options(int argc, char *argv[], char *operation)
 {
   int error= 0;
-  int i= 0;
-  
+
   /* Parse and execute command-line options */
   if ((error= handle_options(&argc, &argv, my_long_options, get_one_option)))
-    goto exit;
+    return error;
 
   /* If the print defaults option used, exit. */
   if (opt_print_defaults)
-  {
-    error= -1;
-    goto exit;
-  }
+    return -1;
 
   /* Add a trailing directory separator if not present */
   if (opt_basedir)
   {
-    i= (int)strlength(opt_basedir);
-    if (opt_basedir[i-1] != FN_LIBCHAR || opt_basedir[i-1] != FN_LIBCHAR2)
+    size_t basedir_len= strlength(opt_basedir);
+    if (opt_basedir[basedir_len - 1] != FN_LIBCHAR ||
+        opt_basedir[basedir_len - 1] != FN_LIBCHAR2)
     {
       char buff[FN_REFLEN];
-      memset(buff, 0, sizeof(buff));
-      
-      strncpy(buff, opt_basedir, sizeof(buff) - 1);
-#ifdef _WIN32
-      strncat(buff, "/", sizeof(buff) - strlen(buff) - 1);
-#else
-      strncat(buff, FN_DIRSEP, sizeof(buff) - strlen(buff) - 1);
-#endif
-      buff[sizeof(buff) - 1]= 0;
+      if (basedir_len + 2 > FN_REFLEN)
+        return -1;
+
+      memcpy(buff, opt_basedir, basedir_len);
+      buff[basedir_len]= '/';
+      buff[basedir_len + 1]= '\0';
+
       my_free(opt_basedir);
       opt_basedir= my_strdup(PSI_NOT_INSTRUMENTED, buff, MYF(MY_FAE));
     }
   }
-  
+
   /*
     If the user did not specify the option to skip loading defaults from a
     config file and the required options are not present or there was an error
     generated when the defaults were read from the file, exit.
   */
   if (!opt_no_defaults && ((error= get_default_values())))
-  {
-    error= -1;
-    goto exit;
-  }
+    return -1;
 
   /*
    Check to ensure required options are present and validate the operation.
@@ -906,11 +905,9 @@ static int process_options(int argc, char *argv[], char *operation)
    read a configuration file named <plugin_name>.ini from the --plugin-dir
    or --plugin-ini location if the --plugin-ini option presented.
   */
-  strcpy(operation, "");
-  if ((error = check_options(argc, argv, operation)))
-  {
-    goto exit;
-  }
+  operation[0]= '\0';
+  if ((error= check_options(argc, argv, operation)))
+    return error;
 
   if (opt_verbose)
   {
@@ -922,25 +919,24 @@ static int process_options(int argc, char *argv[], char *operation)
       printf("# lc_messages_dir = %s\n", opt_lc_messages_dir);
   }
 
-exit:
-  return error;
+  return 0;
 }
 
 
 /**
   Check access
-  
+
   This method checks to ensure all of the directories (opt_basedir,
   opt_plugin_dir, opt_datadir, and opt_plugin_ini) are accessible by
   the user.
-  
+
   @retval int error = 1, success = 0
 */
 
 static int check_access()
 {
   int error= 0;
-  
+
   if ((error= my_access(opt_basedir, F_OK)))
   {
     fprintf(stderr, "ERROR: Cannot access basedir at '%s'.\n",
@@ -1052,13 +1048,13 @@ static int find_plugin(char *tp_path)
 
 /**
   Build the bootstrap file.
-  
+
   Create a new file and populate it with SQL commands to ENABLE or DISABLE
   the plugin via REPLACE and DELETE operations on the mysql.plugin table.
 
   param[in]  operation  The type of operation (ENABLE or DISABLE)
   param[out] bootstrap  A FILE* pointer
-  
+
   @retval int error = 1, success = 0
 */
 
@@ -1066,7 +1062,7 @@ static int build_bootstrap_file(char *operation, char *bootstrap)
 {
   int error= 0;
   FILE *file= 0;
-  
+
   /*
     Perform plugin operation : ENABLE or DISABLE
 
@@ -1077,10 +1073,10 @@ static int build_bootstrap_file(char *operation, char *bootstrap)
     <plugin_name>.ini configuration file. Once the file is built, a call to
     mysqld is made in read only, bootstrap modes to read the SQL statements
     and execute them.
-    
+
     Note: Replace was used so that if a user loads a newer version of a
           library with a different library name, the new library name is
-          used for symbols that match. 
+          used for symbols that match.
   */
   if ((error= make_tempfile(bootstrap, "sql")))
   {
@@ -1127,7 +1123,7 @@ static int build_bootstrap_file(char *operation, char *bootstrap)
       printf("# Disabling %s...\n", plugin_data.name);
     }
   }
-  
+
 exit:
   fclose(file);
   return error;
@@ -1136,11 +1132,11 @@ exit:
 
 /**
   Dump bootstrap file.
-  
+
   Read the contents of the bootstrap file and print it out.
-  
+
   @param[in]  bootstrap_file  Name of bootstrap file to read
-  
+
   @retval int error = 1, success = 0
 */
 
@@ -1177,7 +1173,7 @@ exit:
 
 /**
   Bootstrap the server
-  
+
   Create a command line sequence to launch mysqld in bootstrap mode. This
   will allow mysqld to launch a minimal server instance to read and
   execute SQL commands from a file piped in (the bootstrap file). We use
@@ -1198,47 +1194,39 @@ exit:
 
 static int bootstrap_server(char *server_path, char *bootstrap_file)
 {
-  char bootstrap_cmd[FN_REFLEN];
+  char bootstrap_cmd[FN_REFLEN]= {0};
+  char lc_messages_dir_str[FN_REFLEN]= {0};
   int error= 0;
 
 #ifdef _WIN32
   char *format_str= 0;
   const char *verbose_str= NULL;
-   
-  
+#endif
+
+  if (opt_lc_messages_dir != NULL)
+    snprintf(lc_messages_dir_str, sizeof(lc_messages_dir_str), "--lc-messages-dir=%s",
+             opt_lc_messages_dir);
+
+#ifdef _WIN32
   if (opt_verbose)
     verbose_str= "--console";
   else
     verbose_str= "";
+
   if (has_spaces(opt_datadir) || has_spaces(opt_basedir) ||
-      has_spaces(bootstrap_file))
-  {
-    if (opt_lc_messages_dir != NULL)
-      format_str= "\"%s %s --bootstrap --datadir=%s --basedir=%s --lc-messages-dir=%s <%s\"";
-    else
-    format_str= "\"%s %s --bootstrap --datadir=%s --basedir=%s <%s\"";
-  }
+      has_spaces(bootstrap_file) || has_spaces(lc_messages_dir_str))
+    format_str= "\"%s %s --bootstrap --datadir=%s --basedir=%s %s <%s\"";
   else
-  {
-    if (opt_lc_messages_dir != NULL)
-      format_str= "\"%s %s --bootstrap --datadir=%s --basedir=%s --lc-messages-dir=%s <%s\"";
-    else
-      format_str= "%s %s --bootstrap --datadir=%s --basedir=%s <%s";
-  }
+    format_str= "%s %s --bootstrap --datadir=%s --basedir=%s %s <%s";
+
   snprintf(bootstrap_cmd, sizeof(bootstrap_cmd), format_str,
            add_quotes(convert_path(server_path)), verbose_str,
            add_quotes(opt_datadir), add_quotes(opt_basedir),
-           add_quotes(bootstrap_file));
+           add_quotes(lc_messages_dir_str), add_quotes(bootstrap_file));
 #else
-  if (opt_lc_messages_dir != NULL)
-    snprintf(bootstrap_cmd, sizeof(bootstrap_cmd),
-             "%s --no-defaults --bootstrap --datadir=%s --basedir=%s --lc-messages-dir=%s"
-             " <%s", server_path, opt_datadir, opt_basedir, opt_lc_messages_dir, bootstrap_file);
-  else
-    snprintf(bootstrap_cmd, sizeof(bootstrap_cmd),
-             "%s --no-defaults --bootstrap --datadir=%s --basedir=%s"
-             " <%s", server_path, opt_datadir, opt_basedir, bootstrap_file);
-
+  snprintf(bootstrap_cmd, sizeof(bootstrap_cmd),
+           "%s --no-defaults --bootstrap --datadir=%s --basedir=%s %s"
+           " <%s", server_path, opt_datadir, opt_basedir, lc_messages_dir_str, bootstrap_file);
 #endif
 
   /* Execute the command */
@@ -1251,6 +1239,6 @@ static int bootstrap_server(char *server_path, char *bootstrap_file)
     fprintf(stderr,
             "ERROR: Unexpected result from bootstrap. Error code: %d.\n",
             error);
-  
+
   return error;
 }
