@@ -32,12 +32,11 @@ then
   # build is not running on Gitlab-CI.
   sed '/-DPLUGIN_COLUMNSTORE=NO/d' -i debian/rules
   # Take the files and part of control from MCS directory
-  if [ ! -f debian/mariadb-plugin-columnstore.install ]
-  then
-    cp -v storage/columnstore/columnstore/debian/mariadb-plugin-columnstore.* debian/
-    echo >> debian/control
-    sed "s/-10.6//" <storage/columnstore/columnstore/debian/control >> debian/control
-  fi
+  cp -v storage/columnstore/columnstore/debian/mariadb-plugin-columnstore.* debian/
+  # idempotent, except for the blank line, but that can be tolerated.
+  sed -e '/Package: mariadb-plugin-columnstore/,/^$/d' -i debian/control
+  echo >> debian/control
+  sed "s/-10.6//" <storage/columnstore/columnstore/debian/control >> debian/control
 fi
 
 # Look up distro-version specific stuff
@@ -72,19 +71,20 @@ replace_uring_with_aio()
       -e '/-DWITH_URING=ON/d' -i debian/rules
 }
 
-disable_pmem()
-{
-  sed '/libpmem-dev/d' -i debian/control
-  sed '/-DWITH_PMEM=ON/d' -i debian/rules
-}
-
 disable_libfmt()
 {
   # 7.0+ required
   sed '/libfmt-dev/d' -i debian/control
 }
 
+remove_package_notes()
+{
+  # binutils >=2.39 + disto makefile /usr/share/debhelper/dh_package_notes/package-notes.mk
+  sed -e '/package.notes/d' -i debian/rules debian/control
+}
+
 architecture=$(dpkg-architecture -q DEB_BUILD_ARCH)
+uname_machine=$(uname -m)
 
 # Parse release name and number from Linux standard base release
 # Example:
@@ -117,21 +117,14 @@ in
   "buster")
     disable_libfmt
     replace_uring_with_aio
-    if [ ! "$architecture" = amd64 ]
-    then
-      disable_pmem
-    fi
     ;&
   "bullseye")
     add_lsb_base_depends
+    remove_package_notes
     ;&
   "bookworm")
     # mariadb-plugin-rocksdb in control is 4 arches covered by the distro rocksdb-tools
     # so no removal is necessary.
-    if [[ ! "$architecture" =~ amd64|arm64|ppc64el ]]
-    then
-      disable_pmem
-    fi
     if [[ ! "$architecture" =~ amd64|arm64|armel|armhf|i386|mips64el|mipsel|ppc64el|s390x ]]
     then
       replace_uring_with_aio
@@ -148,12 +141,9 @@ in
     ;&
   "jammy"|"kinetic")
     add_lsb_base_depends
+    remove_package_notes
     ;&
   "lunar"|"mantic")
-    if [[ ! "$architecture" =~ amd64|arm64|ppc64el ]]
-    then
-      disable_pmem
-    fi
     if [[ ! "$architecture" =~ amd64|arm64|armhf|ppc64el|s390x ]]
     then
       replace_uring_with_aio
@@ -201,11 +191,19 @@ fi
 
 # Use eatmydata is available to build faster with less I/O, skipping fsync()
 # during the entire build process (safe because a build can always be restarted)
-if which eatmydata > /dev/null
+if command -v eatmydata > /dev/null
 then
   BUILDPACKAGE_DPKGCMD+=("eatmydata")
 fi
 
+# If running autobake-debs.sh inside docker/podman host machine which
+# has 64 bits cpu but container image is 32 bit make sure that we set
+# correct arch with linux32 for 32 bit enviroment
+if [ "$architecture" = "i386" ] && [ "$uname_machine" = "x86_64" ]
+then
+  BUILDPACKAGE_DPKGCMD+=("linux32")
+fi
+
 BUILDPACKAGE_DPKGCMD+=("dpkg-buildpackage")
 
 # Using dpkg-buildpackage args
