@@ -1,6 +1,6 @@
 /* iotsafe.c
  *
- * Copyright (C) 2006-2021 wolfSSL Inc.
+ * Copyright (C) 2006-2024 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -241,7 +241,7 @@ static int iotsafe_cmd_add_tlv_ex(char *cmd, byte tag, uint16_t len,
     char *len_csim_str = (cmd + AT_CSIM_CMD_SIZE - 5);
     word32 cmdlen;
 
-    cmdlen = XSTRLEN(cmd);
+    cmdlen = (word32)XSTRLEN(cmd);
     if (cmdlen < AT_CSIM_CMD_SIZE) {
         return BAD_FUNC_ARG;
     }
@@ -261,7 +261,7 @@ static int iotsafe_cmd_add_tlv_ex(char *cmd, byte tag, uint16_t len,
     cur_csim_len += (len_csim_str[1] - '0') * 10;
     if (len_csim_str[0] >= '0' &&
             len_csim_str[0] <= '9') {
-        cur_csim_len += len_csim_str[0] * 100;
+        cur_csim_len += (len_csim_str[0] - '0') * 100;
     } else if (len_csim_str[0] != ' ') {
         return BAD_FUNC_ARG;
     }
@@ -440,9 +440,13 @@ static int iotsafe_readfile(uint8_t *file_id, uint16_t file_id_sz,
 {
     char *resp;
     int ret;
-    char *filesz_s;
-    int file_sz = 0;
     uint16_t off = 0;
+#ifdef IOTSAFE_NO_GETDATA
+    int file_sz = max_size;
+#else
+    int file_sz = 0;
+    char *filesz_s;
+
     iotsafe_cmd_start(csim_cmd, IOTSAFE_CLASS, IOTSAFE_INS_GETDATA,
             IOTSAFE_GETDATA_FILE, 0);
     iotsafe_cmd_add_tlv(csim_cmd, IOTSAFE_TAG_FILE_ID, file_id_sz, file_id);
@@ -471,6 +475,7 @@ static int iotsafe_readfile(uint8_t *file_id, uint16_t file_id_sz,
         WOLFSSL_MSG("iotsafe_readfile: insufficient space in buffer");
         return -1;
     }
+#endif
 
     while (off < file_sz) {
         byte off_p1, off_p2;
@@ -488,11 +493,29 @@ static int iotsafe_readfile(uint8_t *file_id, uint16_t file_id_sz,
                 return -1;
             }
             off += ret/2;
+#ifdef IOTSAFE_NO_GETDATA
+            if (XSTRNCMP(&resp[ret-4], "0000", 4) == 0) {
+                /* Strip trailing zeros */
+                int idx = 0;
+                for (idx = 0; idx < off-1; idx+=2) {
+                    if (content[idx] == 0 && content[idx+1] == 0) {
+                        off = idx;
+#ifdef DEBUG_IOTSAFE
+                        WOLFSSL_MSG("Stripped trailing zeros from cert buffer.");
+                        WOLFSSL_BUFFER(content, off);
+#endif
+                        break;
+                    }
+                }
+                break;
+            }
+#endif
         } else {
             WOLFSSL_MSG("IoTSafe: Error reading file.");
             return -1;
         }
     }
+
     return off;
 }
 
@@ -589,7 +612,7 @@ static int iotsafe_parse_public_key(char* resp, int len, ecc_key *key)
 /* Execute GEN_KEYPAIR on the IoT-SAFE applet.
  *
  * Return -1 on error; 0 if the operation is successful, but
- * the generated public key was not yet stored in `key`; 1 if 
+ * the generated public key was not yet stored in `key`; 1 if
  * the operation is successful and the public key was found in the
  * command response and copied to the `key` structure, if not NULL.
  */
@@ -597,7 +620,7 @@ static int iotsafe_gen_keypair(byte *wr_slot, unsigned long id_size,
                                ecc_key *key)
 {
     char *resp;
-    int ret = WC_HW_E;
+    int ret = WC_NO_ERR_TRACE(WC_HW_E);
     iotsafe_cmd_start(csim_cmd, IOTSAFE_CLASS, IOTSAFE_INS_GEN_KEYPAIR, 0, 0);
     iotsafe_cmd_add_tlv(csim_cmd, IOTSAFE_TAG_PRIVKEY_ID, id_size, wr_slot);
     iotsafe_cmd_complete(csim_cmd);
@@ -719,6 +742,79 @@ static int iotsafe_put_public_key(byte *pubkey_id, unsigned long id_size,
     }
     return ret;
 }
+#ifdef HAVE_HKDF
+static int iotsafe_hkdf_extract(byte* prk, const byte* salt, word32 saltLen,
+       byte* ikm, word32 ikmLen, int digest)
+{
+    int ret;
+    char *resp;
+    uint16_t hash_algo = 0;
+    int hash_len;
+    uint16_t hash_algo_be = 0;
+
+    WOLFSSL_MSG("Enter iotsafe_hkdf_extract");
+    switch (digest) {
+#ifndef NO_SHA256
+        case WC_SHA256:
+            hash_algo = (uint16_t)1;
+            hash_len = WC_SHA256_DIGEST_SIZE;
+            break;
+#endif
+#ifdef WOLFSSL_SHA384
+        case WC_SHA384:
+            hash_algo = (uint16_t)2;
+            hash_len = WC_SHA384_DIGEST_SIZE;
+            break;
+#endif
+#ifdef WOLFSSL_SHA512
+        case WC_SHA512:
+            hash_algo = (uint16_t)4;
+            hash_len = WC_SHA512_DIGEST_SIZE;
+            break;
+#endif
+        default:
+            return BAD_FUNC_ARG;
+            break;
+    }
+
+    if (ikmLen == 0) {
+        ikmLen = hash_len;
+        XMEMSET(ikm, 0, hash_len);
+    }
+
+#ifdef DEBUG_IOTSAFE
+    printf("IOTSAFE PK HKDF Extract\n");
+    printf("salt: ");
+    for(word32 i = 0; i < saltLen; i++)
+        printf("%02X", salt[i]);
+
+    printf("\nikm: ");
+    for(word32 i = 0; i < ikmLen; i++)
+        printf("%02X", ikm[i]);
+
+    printf("\nhash: %d\n", digest);
+#endif
+
+    hash_algo_be = XHTONS(hash_algo);
+
+    iotsafe_cmd_start(csim_cmd, IOTSAFE_CLASS, IOTSAFE_INS_HKDF_EXTRACT, 0, 0);
+    iotsafe_cmd_add_tlv(csim_cmd, IOTSAFE_TAG_SECRET, ikmLen, ikm);
+    iotsafe_cmd_add_tlv(csim_cmd, IOTSAFE_TAG_SALT, saltLen,salt);
+    iotsafe_cmd_add_tlv(csim_cmd, IOTSAFE_TAG_HASH_ALGO, 2, (const byte*)&hash_algo_be);
+    iotsafe_cmd_complete(csim_cmd);
+    if (expect_csim_response(csim_cmd, (word32)XSTRLEN(csim_cmd), &resp) < 1) {
+        WOLFSSL_MSG("Unexpected reply from HKDF extract");
+        ret = WC_HW_E;
+    } else {
+        ret = hexbuffer_conv(resp, prk, hash_len);
+        if (ret < 0)
+            ret = WC_HW_E;
+        else
+            ret = 0;
+    }
+    return ret;
+}
+#endif
 
 static int iotsafe_sign_hash(byte *privkey_idx, uint16_t id_size,
         uint16_t hash_algo, uint8_t sign_algo, const byte *hash, word32 hashLen,
@@ -726,7 +822,7 @@ static int iotsafe_sign_hash(byte *privkey_idx, uint16_t id_size,
 {
     byte mode_of_operation = IOTSAFE_MOO_SIGN_ONLY;
     uint16_t hash_algo_be = XHTONS(hash_algo);
-    int ret = WC_HW_E;
+    int ret = WC_NO_ERR_TRACE(WC_HW_E);
     char *resp;
     char R[2 * IOTSAFE_ECC_KSIZE + 1];
     char S[2 * IOTSAFE_ECC_KSIZE + 1];
@@ -778,7 +874,17 @@ static int iotsafe_sign_hash(byte *privkey_idx, uint16_t id_size,
             byte sig_hdr[3];
             if (hex_to_bytes(resp, sig_hdr, 3) < 0) {
                ret = BAD_FUNC_ARG;
-            } else if ((sig_hdr[0] == IOTSAFE_TAG_SIGNATURE_FIELD) &&
+            }
+#ifdef IOTSAFE_SIG_8BIT_LENGTH
+            else if ((sig_hdr[0] == IOTSAFE_TAG_SIGNATURE_FIELD) &&
+                       (sig_hdr[1] == 2 * IOTSAFE_ECC_KSIZE)) {
+                XSTRNCPY(R, resp + 4, IOTSAFE_ECC_KSIZE * 2);
+                XSTRNCPY(S, resp + 4 + IOTSAFE_ECC_KSIZE * 2,
+                        IOTSAFE_ECC_KSIZE * 2);
+                ret = wc_ecc_rs_to_sig(R, S, signature, sigLen);
+            }
+#endif
+            else if ((sig_hdr[0] == IOTSAFE_TAG_SIGNATURE_FIELD) &&
                        (sig_hdr[1] == 0) &&
                        (sig_hdr[2] == 2 * IOTSAFE_ECC_KSIZE)) {
                 XSTRNCPY(R, resp + 6, IOTSAFE_ECC_KSIZE * 2);
@@ -787,9 +893,10 @@ static int iotsafe_sign_hash(byte *privkey_idx, uint16_t id_size,
                 ret = wc_ecc_rs_to_sig(R, S, signature, sigLen);
             } else {
                 ret = WC_HW_E;
+                WOLFSSL_MSG("Invalid response from EC sign update");
             }
         } else {
-            WOLFSSL_MSG("Invalid response from EC sign update");
+            WOLFSSL_MSG("Invalid/no response from EC sign update");
         }
 
         /* Terminate sign/sign session. */
@@ -960,6 +1067,34 @@ static int wolfIoT_ecc_keygen(WOLFSSL* ssl, struct ecc_key* key,
     return ret;
 }
 
+#ifdef HAVE_HKDF
+static int wolfIoT_hkdf_extract(byte* prk, const byte* salt, word32 saltLen,
+       byte* ikm, word32 ikmLen, int digest, void* ctx)
+{
+    int ret;
+    const  byte* localSalt;  /* either points to user input or tmp */
+    byte tmp[WC_MAX_DIGEST_SIZE]; /* localSalt helper */
+
+    WOLFSSL_MSG("IOTSAFE: Called wolfIoT_hkdf_extract\n");
+
+    localSalt = salt;
+
+    if (saltLen == 0) {
+        ret = wc_HmacSizeByType(digest);
+            if (ret < 0)
+                return ret;
+            saltLen = ret;
+            if (localSalt == NULL) {
+                XMEMSET(tmp, 0, saltLen);
+                localSalt = tmp;
+            }
+    }
+
+    ret = iotsafe_hkdf_extract(prk, localSalt, saltLen, ikm, ikmLen, digest);
+    (void)ctx;
+    return ret;
+}
+#endif
 
 static int wolfIoT_ecc_sign(WOLFSSL* ssl,
        const unsigned char* in, unsigned int inSz,
@@ -1429,13 +1564,16 @@ int wolfSSL_CTX_iotsafe_enable(WOLFSSL_CTX *ctx)
     wolfSSL_CTX_SetEccVerifyCb(ctx, wolfIoT_ecc_verify);
     wolfSSL_CTX_SetEccKeyGenCb(ctx, wolfIoT_ecc_keygen);
     wolfSSL_CTX_SetEccSharedSecretCb(ctx, wolfIoT_ecc_shared_secret);
+    #ifdef HAVE_HKDF
+    wolfSSL_CTX_SetHKDFExtractCb(ctx, wolfIoT_hkdf_extract);
+    #endif
     #ifndef NO_DH
     wolfSSL_CTX_SetDhAgreeCb(ctx, wolfIoT_dh_agree);
     #endif /* NO_DH */
     WOLFSSL_MSG("ECC callbacks set to IoT_safe interface");
     #endif
     #ifndef NO_RSA
-    /* wolfSSL_CTX_SetRsaSignCb(wolfIoT_rsa_sign);  // TODO: RSA callbacks */
+    /* wolfSSL_CTX_SetRsaSignCb(wolfIoT_rsa_sign); */ /* TODO: RSA callbacks */
     #endif
 #else
     (void)ctx;
