@@ -447,7 +447,8 @@ enum my_lex_states
   MY_LEX_IDENT_OR_KEYWORD,
   MY_LEX_IDENT_OR_HEX, MY_LEX_IDENT_OR_BIN, MY_LEX_IDENT_OR_NCHAR,
   MY_LEX_STRING_OR_DELIMITER, MY_LEX_MINUS_OR_COMMENT, MY_LEX_PLACEHOLDER,
-  MY_LEX_COMMA
+  MY_LEX_COMMA,
+  MY_LEX_IDENT_OR_QUALIFIED_SPECIAL_FUNC
 };
 
 struct charset_info_st;
