@@ -2,6 +2,15 @@
 
 set -e
 
+if [ -n "$BENCH" ] ; then
+  sudo benchmark/build.sh
+  cd benchmark
+  sudo ./installation.sh
+  sudo ./launch.sh
+  exit
+fi
+
+export CC_DIR=/home/travis/build/mariadb-corporation/mariadb-connector-c
 if [ -n "$server_branch" ] ; then
 
   ###################################################################################################################
@@ -14,32 +23,47 @@ if [ -n "$server_branch" ] ; then
   sudo tail /etc/hosts
 
   # get latest server
-  git clone -b ${server_branch} https://github.com/mariadb/server ../workdir-server
+  git clone -b ${server_branch} https://github.com/mariadb/server ../workdir-server --depth=1
 
   cd ../workdir-server
+  export SERVER_DIR=$PWD
+
   # don't pull in submodules. We want the latest C/C as libmariadb
   # build latest server with latest C/C as libmariadb
   # skip to build some storage engines to speed up the build
-  cmake -DPLUGIN_MROONGA=NO -DPLUGIN_ROCKSDB=NO -DPLUGIN_SPIDER=NO -DPLUGIN_TOKUDB=NO
-  cd libmariadb
-    echo "PR:${TRAVIS_PULL_REQUEST} TRAVIS_COMMIT:${TRAVIS_COMMIT}"
+
   if [ -n "$TRAVIS_PULL_REQUEST" ] && [ "$TRAVIS_PULL_REQUEST" != "false" ] ; then
-    # fetching pull request
-    echo "fetching PR"
-    git fetch origin pull/${TRAVIS_PULL_REQUEST}/head:PR_${TRAVIS_PULL_REQUEST}
-    echo "checkout PR"
-    git checkout PR_${TRAVIS_PULL_REQUEST}
+    git submodule update --init --remote libmariadb
+    cd libmariadb
+    git fetch origin ${TRAVIS_PULL_REQUEST}
+    git checkout -qf FETCH_HEAD
   else
-    echo "checkout commit"
+    git submodule set-branch -b ${TRAVIS_BRANCH} libmariadb
+    git submodule sync
+    git submodule update --init --remote libmariadb
+    cd libmariadb
     git checkout ${TRAVIS_COMMIT}
   fi
 
-  cd ..
+  cd $SERVER_DIR
   git add libmariadb
+
+  mkdir bld
+  cd bld
+  cmake .. -DPLUGIN_MROONGA=NO -DPLUGIN_ROCKSDB=NO -DPLUGIN_SPIDER=NO -DPLUGIN_TOKUDB=NO
+  echo "PR:${TRAVIS_PULL_REQUEST} TRAVIS_COMMIT:${TRAVIS_COMMIT}"
+  if [ -n "$TRAVIS_PULL_REQUEST" ] && [ "$TRAVIS_PULL_REQUEST" != "false" ] ; then
+    # fetching pull request
+    echo "fetching PR"
+  else
+    echo "checkout commit"
+  fi
+
+  cd $SERVER_DIR/bld
   make -j9
 
   cd mysql-test/
-  ./mysql-test-run.pl --suite=main ${TEST_OPTION} --parallel=auto --skip-test=session_tracker_last_gtid
+  ./mysql-test-run.pl --suite=main,unit ${TEST_OPTION} --parallel=auto --skip-test=session_tracker_last_gtid
 
 else
 
@@ -48,18 +72,19 @@ else
   ###################################################################################################################
   echo "run connector test suite"
 
-  cmake . -DCMAKE_BUILD_TYPE=RelWithDebInfo -DCERT_PATH=${SSLCERT}
+  mkdir bld
+  cd bld
+  cmake .. -DCMAKE_BUILD_TYPE=RelWithDebInfo -DCERT_PATH=${SSLCERT}
 
   if [ "$TRAVIS_OS_NAME" = "windows" ] ; then
     echo "build from windows"
-    set MARIADB_CC_TEST=1
-    set MYSQL_TEST_DB=testc
-    set MYSQL_TEST_TLS=%TEST_REQUIRE_TLS%
-    set MYSQL_TEST_USER=%TEST_DB_USER%
-    set MYSQL_TEST_HOST=%TEST_DB_HOST%
-    set MYSQL_TEST_PASSWD=%TEST_DB_PASSWORD%
-    set MYSQL_TEST_PORT=%TEST_DB_PORT%
-    set MYSQL_TEST_TLS=%TEST_REQUIRE_TLS%
+    export MARIADB_CC_TEST=1
+    export MYSQL_TEST_DB=testc
+    export MYSQL_TEST_TLS=$TEST_REQUIRE_TLS
+    export MYSQL_TEST_USER=$TEST_DB_USER
+    export MYSQL_TEST_HOST=$TEST_DB_HOST
+    export MYSQL_TEST_PASSWD=$TEST_DB_PASSWORD
+    export MYSQL_TEST_PORT=$TEST_DB_PORT
     cmake --build . --config RelWithDebInfo
   else
     echo "build from linux"
@@ -79,8 +104,8 @@ else
     fi
     export MYSQL_TEST_TLS=$TEST_REQUIRE_TLS
     export SSLCERT=$TEST_DB_SERVER_CERT
-    if [ -n "$MYSQL_TEST_SSL_PORT" ] ; then
-      export MYSQL_TEST_SSL_PORT=$MYSQL_TEST_SSL_PORT
+    if [ -n "$TEST_MAXSCALE_TLS_PORT" ] ; then
+      export MYSQL_TEST_SSL_PORT=$TEST_MAXSCALE_TLS_PORT
     fi
     make
   fi
