@@ -49,6 +49,7 @@ static int test_conc66(MYSQL *my)
   fprintf(fp, "user=conc66\n");
   fprintf(fp, "port=3306\n");
   fprintf(fp, "enable-local-infile\n");
+  fprintf(fp, "server_plugin=file_key_management:file_key_management_algorithm=AES_CTR;file_key_management_key=secret\n");
   fprintf(fp, "password='test@A1\\\";#test'\n");
 
   fclose(fp);
@@ -541,8 +542,6 @@ static int test_compress(MYSQL *mysql)
   /* use compressed protocol */
   rc= mysql_options(mysql, MYSQL_OPT_COMPRESS, NULL);
 
-
-
   if (!(my_test_connect(mysql, hostname, username,
                            password, schema, port,
                            socketname, 0)))
@@ -618,6 +617,12 @@ int test_conc21(MYSQL *mysql)
   int major=0, minor= 0, patch=0;
   SKIP_MAXSCALE;
 
+  if (strlen(mysql_get_server_info(mysql)) > 63)
+  {
+    diag("server name is too long - skip until rpl hack was removed");
+    return SKIP;
+  }
+
   rc= mysql_query(mysql, "SELECT @@version");
   check_mysql_rc(rc, mysql);
 
@@ -642,14 +647,14 @@ int test_conc26(MYSQL *unused __attribute__((unused)))
   MYSQL *mysql= mysql_init(NULL);
   mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "ascii");
 
-  FAIL_IF(my_test_connect(mysql, hostname, "notexistinguser", "password", schema, port, NULL, CLIENT_REMEMBER_OPTIONS), 
+  FAIL_IF(my_test_connect(mysql, hostname, "notexistinguser", "password", schema, port, socketname, CLIENT_REMEMBER_OPTIONS),
           "Error expected");
   FAIL_IF(!mysql->options.charset_name || strcmp(mysql->options.charset_name, "ascii") != 0,
           "expected charsetname=ascii");
   mysql_close(mysql);
 
   mysql= mysql_init(NULL);
-  FAIL_IF(my_test_connect(mysql, hostname, "notexistinguser", "password", schema, port, NULL, 0), 
+  FAIL_IF(my_test_connect(mysql, hostname, "notexistinguser", "password", schema, port, socketname, 0), 
           "Error expected");
   FAIL_IF(mysql->options.charset_name, "Error: options not freed");
   mysql_close(mysql);
@@ -664,7 +669,7 @@ int test_connection_timeout(MYSQL *unused __attribute__((unused)))
   MYSQL *mysql= mysql_init(NULL);
   mysql_options(mysql, MYSQL_OPT_CONNECT_TIMEOUT, (unsigned int *)&timeout);
   start= time(NULL);
-  if (my_test_connect(mysql, "192.168.1.101", "notexistinguser", "password", schema, port, NULL, CLIENT_REMEMBER_OPTIONS))
+  if (my_test_connect(mysql, "192.168.1.101", "notexistinguser", "password", schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
   {
     diag("Error expected - maybe you have to change hostname");
     return FAIL;
@@ -684,13 +689,16 @@ int test_connection_timeout2(MYSQL *unused __attribute__((unused)))
 
   SKIP_SKYSQL;
   SKIP_MAXSCALE;
+  SKIP_TLS;
 
   mysql= mysql_init(NULL);
   mysql_options(mysql, MYSQL_OPT_CONNECT_TIMEOUT, (unsigned int *)&timeout);
-  mysql_options(mysql, MYSQL_INIT_COMMAND, "set @a:=SLEEP(6)");
+  mysql_options(mysql, MYSQL_INIT_COMMAND, "set @a:=SLEEP(7)");
   start= time(NULL);
-  if (my_test_connect(mysql, hostname, username, password, schema, port, NULL, CLIENT_REMEMBER_OPTIONS))
+  if (my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
   {
+  elapsed= time(NULL) - start;
+  diag("elapsed: %lu", (unsigned long)elapsed);
     diag("timeout error expected");
     return FAIL;
   }
@@ -1071,7 +1079,7 @@ static int test_unix_socket_close(MYSQL *unused __attribute__((unused)))
 
   for (i=0; i < 10000; i++)
   {
-    my_test_connect(mysql, "localhost", "user", "passwd", NULL, 0, "./dummy_sock", 0);
+    mysql_real_connect(mysql, "localhost", "user", "passwd", NULL, 0, "./dummy_sock", 0);
     /* check if we run out of sockets */
     if (mysql_errno(mysql) == 2001)
     {
@@ -1153,6 +1161,9 @@ static int test_auth256(MYSQL *my)
   if (IS_SKYSQL(hostname))
     return SKIP;
 
+  // xpand doesn't have information_schema.plugins
+  SKIP_XPAND;
+
   if (!mysql_client_find_plugin(mysql, "sha256_password", MYSQL_CLIENT_AUTHENTICATION_PLUGIN))
   {
     diag("sha256_password plugin not available");
@@ -1194,6 +1205,7 @@ static int test_auth256(MYSQL *my)
   if (!my_test_connect(mysql, hostname, "sha256user", "foo", NULL, port, socketname, 0))
   {
     diag("error: %s", mysql_error(mysql));
+    diag("host: %s", this_host);
     mysql_close(mysql);
     return FAIL;
   }
@@ -1210,6 +1222,9 @@ static int test_mdev13100(MYSQL *my __attribute__((unused)))
   int rc;
   FILE *fp;
 
+  /* MXS-4898: MaxScale sends utf8mb4 in handshake OK packet */
+  SKIP_MAXSCALE;
+
   if (!(fp= fopen("./mdev13100.cnf", "w")))
     return FAIL;
 
@@ -1228,6 +1243,7 @@ static int test_mdev13100(MYSQL *my __attribute__((unused)))
     diag("Error: %s", mysql_error(mysql));
     return FAIL;
   }
+  diag("Default charset: %s", mysql_character_set_name(mysql));
   FAIL_IF(strcmp("latin2", mysql_character_set_name(mysql)), "Expected charset latin2");
   mysql_close(mysql);
 
@@ -1663,6 +1679,7 @@ static int test_conc312(MYSQL *my)
 
   if (rc)
   {
+    diag("Error: %s", mysql_error(my));
     diag("caching_sha256_password not supported");
     return SKIP; 
   }
@@ -1869,6 +1886,8 @@ static int test_gtid(MYSQL *mysql)
 
   if (is_mariadb)
     return SKIP;
+  // https://jira.mariadb.org/browse/XPT-182
+  SKIP_XPAND;
 
   rc= mysql_query(mysql, "SET @@session.session_track_state_change=1");
   check_mysql_rc(rc, mysql);
@@ -1915,7 +1934,575 @@ static int test_conc490(MYSQL *my __attribute__((unused)))
   return OK;
 }
 
+static int test_conc544(MYSQL *mysql)
+{
+  int rc;
+  MYSQL *my= mysql_init(NULL);
+  char query[1024];
+
+  SKIP_SKYSQL;
+  SKIP_MAXSCALE;
+
+  if (!mysql_client_find_plugin(mysql, "client_ed25519", MYSQL_CLIENT_AUTHENTICATION_PLUGIN))
+  {
+    diag("client_ed25519 plugin not available");
+    return SKIP;
+  }
+
+  rc= mysql_query(mysql, "INSTALL SONAME 'auth_ed25519'");
+  if (rc)
+  {
+    diag("feature not supported, ed25519 plugin not available");
+    return SKIP;
+  }
+
+  rc= mysql_optionsv(my, MARIADB_OPT_RESTRICTED_AUTH, "client_ed25519");
+  check_mysql_rc(rc, mysql);
+
+  if (my_test_connect(my, hostname, username,
+                             password, schema, port, socketname, 0))
+  {
+    diag("error expected (restricted auth)");
+    return FAIL;
+  }
+  mysql_close(my);
+
+  if (mysql_get_server_version(mysql) < 100400) {
+    sprintf(query, "CREATE OR REPLACE USER 'ede'@'%s' IDENTIFIED VIA ed25519 USING '6aW9C7ENlasUfymtfMvMZZtnkCVlcb1ssxOLJ0kj/AA'", this_host);
+  } else {
+    sprintf(query, "CREATE OR REPLACE USER 'ede'@'%s' IDENTIFIED VIA ed25519 USING PASSWORD('MySup8%%rPassw@ord')", this_host);
+  }
+  rc= mysql_query(mysql, query);
+  check_mysql_rc(rc, mysql);
+
+  sprintf(query, "GRANT ALL ON %s.* TO 'ede'@'%s'", schema, this_host);
+  rc= mysql_query(mysql, query);
+  check_mysql_rc(rc, mysql);
+
+  my= mysql_init(NULL);
+  if (plugindir)
+    mysql_optionsv(my, MYSQL_PLUGIN_DIR, plugindir);
+  mysql_optionsv(my, MARIADB_OPT_RESTRICTED_AUTH, "client_ed25519, mysql_native_password");
+  if (!my_test_connect(my, hostname, "ede", "MySup8%rPassw@ord", schema, port, socketname, 0))
+  {
+    diag("Error: %s", mysql_error(my));
+    return FAIL;
+  }
+  mysql_close(my);
+
+  sprintf(query, "DROP USER 'ede'@'%s'", this_host);
+  rc= mysql_query(mysql, query);
+  check_mysql_rc(rc, mysql);
+
+  sprintf(query, "UNINSTALL SONAME 'auth_ed25519'");
+  rc= mysql_query(mysql, query);
+  check_mysql_rc(rc, mysql);
+
+  return OK;
+}
+
+static int test_conn_str(MYSQL *my __attribute__((unused)))
+{
+  MYSQL *mysql= mysql_init(NULL);
+  char conn_str[1024];
+  int rc=OK;
+
+  snprintf(conn_str, sizeof(conn_str)-1, "host=%s;user=%s;password={%s};port=%d;socket=%s",
+                hostname ? hostname : "localhost", username ? username : "", 
+                password ? password : "", 
+                port, socketname ? socketname : "");
+
+  /* SkySQL requires secure connection */
+  if (IS_SKYSQL(hostname))
+  {
+    strcat(conn_str, ";ssl_enforce=1");
+  }
+
+  if (mariadb_connect(mysql, conn_str))
+  {
+    diag("host: %s", mysql->host);
+    diag("user: %s", mysql->user);
+    diag("cipher: %s", mysql_get_ssl_cipher(mysql));
+  } else
+  {
+    diag("error: %s", mysql_error(mysql));
+    rc= FAIL;
+  }
+  mysql_close(mysql);
+  return rc;
+}
+
+static int test_conn_str_1(MYSQL *my __attribute__((unused)))
+{
+  MYSQL *mysql;
+  FILE *fp;
+  int rc;
+  char conn_str[1024];
+  mysql= mysql_init(NULL);
+
+  if (!(fp= fopen("./conc274.cnf", "w")))
+    return FAIL;
+
+  sprintf(conn_str, "connection=host=%s;user=%s;password=%s;port=%d;ssl_enforce=1;socket=%s",
+                hostname ? hostname : "localhost", username ? username : "", 
+                password ? password : "", ssl_port, socketname ? socketname : "");
+
+  fprintf(fp, "[client]\n");
+  fprintf(fp, "%s\n", conn_str);
+
+  fclose(fp);
+
+  rc= mysql_options(mysql, MYSQL_READ_DEFAULT_FILE, "./conc274.cnf");
+  check_mysql_rc(rc, mysql);
+  rc= mysql_options(mysql, MYSQL_READ_DEFAULT_GROUP, "");
+  check_mysql_rc(rc, mysql);
+
+  if (!my_test_connect(mysql, NULL, NULL, NULL, NULL, 0, NULL, 0))
+  {
+    diag("Error: %s", mysql_error(mysql));
+    remove("./conc274.cnf");
+    return FAIL;
+  }
+  remove("./conc274.cnf");
+
+  if (!mysql_get_ssl_cipher(mysql))
+  {
+    diag("Error: No TLS connection");
+    return FAIL;
+  }
+  diag("Cipher in use: %s", mysql_get_ssl_cipher(mysql));
+  mysql_close(mysql);
+  return OK;
+}
+
+static int test_conc365(MYSQL *my __attribute__((unused)))
+{
+  int rc= OK;
+  MYSQL *mysql= mysql_init(NULL);
+  char tmp[1024];
+
+  snprintf(tmp, sizeof(tmp) - 1,
+   "host=127.0.0.1:3300,%s;user=%s;password=%s;port=%d;socket=%s",
+   hostname ? hostname : "localhost", username ? username : "", password ? password : "",
+   port, socketname ? socketname : "");
+
+ if (IS_SKYSQL(hostname))
+   strcat(tmp, ";ssl_enforce=1");
+
+ if (!mariadb_connect(mysql, tmp))
+   rc= FAIL;
+
+  mysql_close(mysql);
+
+  if (rc)
+    return rc;
+
+  mysql= mysql_init(NULL);
+  snprintf(tmp, sizeof(tmp) -1, "127.0.0.1:3300,%s:%d", hostname ? hostname : "localhost", port);
+  if (!my_test_connect(mysql, tmp, username,
+                             password, schema, port, socketname, 0))
+  {
+    diag("Error: %s", mysql_error(mysql));
+    rc= FAIL;
+  }
+
+  mysql_close(mysql);
+
+  if (rc)
+    return rc;
+  
+  mysql= mysql_init(NULL);
+  mysql_options(mysql, MARIADB_OPT_HOST, tmp);
+  if (!my_test_connect(mysql, NULL, username,
+                             password, schema, port, socketname, 0))
+  {
+    diag("Error: %s", mysql_error(mysql));
+    rc= FAIL;
+  }
+
+  mysql_close(mysql);
+  return rc;
+}
+
+static int test_conc365_reconnect(MYSQL *my)
+{
+  int rc= OK;
+  MYSQL *mysql= mysql_init(NULL);
+  char tmp[1024];
+  my_bool reconnect= 1;
+  SKIP_MAXSCALE;
+
+  mysql_options(mysql, MYSQL_OPT_RECONNECT, &reconnect);
+
+  if (IS_SKYSQL(hostname))
+  {
+    snprintf(tmp, sizeof(tmp) - 1,
+      "host=127.0.0.1:3300,%s;user=%s;password=%s;port=%d;socket=%s;ssl_enforce=1",
+      hostname ? hostname : "localhost", username ? username : "", password ? password : "",
+      ssl_port, socketname ? socketname : "");
+  } else {
+    snprintf(tmp, sizeof(tmp) - 1,
+      "host=127.0.0.1:3300,%s;user=%s;password=%s;port=%d;socket=%s",
+      hostname ? hostname : "localhost", username ? username : "", password ? password : "",
+      port, socketname ? socketname : "");
+  }
+
+  if (!my_test_connect(mysql, tmp, username,
+                             password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
+  {
+    diag("Error: %s", mysql_error(mysql));
+    rc= FAIL;
+  }
+
+  sprintf(tmp, "KILL %ld", mysql_thread_id(mysql));
+
+  rc= mysql_query(my, tmp);
+  check_mysql_rc(rc, my);
+
+  sleep(3);
+  rc= mysql_ping(mysql);
+  check_mysql_rc(rc, my);
+
+  mysql_close(mysql);
+  return rc;
+}
+
+struct st_callback {
+  char autocommit;
+  char database[64];
+  char charset[64];
+};
+
+void my_status_callback(void *ptr, enum enum_mariadb_status_info type, ...)
+{
+  va_list ap;
+  struct st_callback *data= (struct st_callback *)ptr;
+  va_start(ap, type);
+
+  switch(type) {
+  case STATUS_TYPE:
+    {
+      int status= va_arg(ap, int);
+      data->autocommit= status & SERVER_STATUS_AUTOCOMMIT;
+    }
+    break;
+  case SESSION_TRACK_TYPE:
+    {
+      enum enum_session_state_type track_type= va_arg(ap, enum enum_session_state_type);
+      switch (track_type) {
+      case SESSION_TRACK_SCHEMA:
+        {
+          MARIADB_CONST_STRING *str= va_arg(ap, MARIADB_CONST_STRING *);
+          strncpy(data->database, str->str, str->length);
+          data->database[str->length]= 0;
+        }
+        break;
+      case SESSION_TRACK_SYSTEM_VARIABLES:
+        {
+          MARIADB_CONST_STRING *key= va_arg(ap, MARIADB_CONST_STRING *);
+          MARIADB_CONST_STRING *val= va_arg(ap, MARIADB_CONST_STRING *);
+
+          if (!strncmp(key->str, "character_set_client", key->length))
+          {
+            strncpy(data->charset, val->str, val->length);
+            data->charset[val->length]= 0;
+          }
+        }
+        break;
+      default:
+        break;
+      }
+    }
+  default:
+    break;
+  }
+  va_end(ap);
+}
+
+static int test_status_callback(MYSQL *my __attribute__((unused)))
+{
+  MYSQL *mysql= mysql_init(NULL);
+  char tmp[64];
+  int rc;
+  struct st_callback data= {0,"", ""};
+
+  rc= mysql_optionsv(mysql, MARIADB_OPT_STATUS_CALLBACK, my_status_callback, &data);
+
+  if (!my_test_connect(mysql, hostname, username,
+                      password, NULL, port, socketname, 0))
+  {
+    diag("error1: %s", mysql_error(mysql));
+    return FAIL;
+  }
+
+  rc= mysql_autocommit(mysql, 0);
+  check_mysql_rc(rc, mysql);
+  rc= mysql_autocommit(mysql, 1);
+  check_mysql_rc(rc, mysql);
+
+  if (!data.autocommit)
+  {
+    diag("autocommit not set");
+    return FAIL;
+  }
+  diag("-------------------------");
+
+  sprintf(tmp, "USE %s", schema);
+  rc= mysql_query(mysql, tmp);
+  check_mysql_rc(rc, mysql);
+
+  if (strcmp(data.database, schema))
+  {
+    diag("Expected database: %s instead of %s", schema, data.database);
+    return FAIL;
+  }
+
+  rc= mysql_query(mysql, "SET NAMES latin1");
+  check_mysql_rc(rc, mysql);
+
+  if (strcmp(data.charset, "latin1"))
+  {
+    diag("Expected charset latin1 instead of %s", data.charset);
+    return FAIL;
+  }
+
+  mysql_close(mysql);
+  return OK;
+}
+
+static int test_conc632(MYSQL *my __attribute__((unused)))
+{
+  MYSQL *mysql= mysql_init(NULL);
+  int rc;
+
+  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
+  {
+    diag("Connection failed. Error: %s", mysql_error(mysql));
+    mysql_close(mysql);
+    return FAIL;
+  }
+
+  rc= mysql_query(mysql, "DROP PROCEDURE conc632");
+
+  rc= mysql_query(mysql, "CREATE PROCEDURE conc632() "
+                         "BEGIN "
+                         "  SELECT 1;"
+                         "  SELECT 2;"
+                         "END");
+  check_mysql_rc(rc, mysql);
+
+  rc= mysql_query(mysql, "CALL conc632()");
+  check_mysql_rc(rc, mysql);
+
+  rc= mysql_reset_connection(mysql);
+  check_mysql_rc(rc, mysql);
+
+  rc= mysql_ping(mysql);
+  check_mysql_rc(rc, mysql);
+
+  rc= mysql_query(mysql, "DROP PROCEDURE conc632");
+  check_mysql_rc(rc, mysql);
+
+  mysql_close(mysql);
+  return OK;
+}
+
+static int test_conc505(MYSQL *my __attribute__((unused)))
+{
+  MYSQL *mysql= mysql_init(NULL);
+
+#define CLIENT_DEPRECATE_EOF (1ULL << 24)
+
+  if (my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_DEPRECATE_EOF))
+  {
+    diag("Error expected: Invalid client flag");
+    mysql_close(mysql);
+    return FAIL;
+  }
+  diag("Error (expected): %s", mysql_error(mysql));
+  FAIL_IF(mysql_errno(mysql) != CR_INVALID_CLIENT_FLAG, "Wrong error number");
+  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_MULTI_STATEMENTS | CLIENT_MULTI_RESULTS))
+  {
+    diag("Error: %s", mysql_error(mysql));
+    mysql_close(mysql);
+    return FAIL;
+  }
+
+  mysql_close(mysql);
+  return OK;
+}
+
+#if defined(HAVE_GNUTLS) && GNUTLS_VERSION_NUMBER >= 0x030700 || defined(HAVE_OPENSSL)
+#define HAVE_test_conc748
+static int test_conc748(MYSQL *my __attribute__((unused)))
+{
+  MYSQL *mysql;
+  int i;
+  const char *ciphers[3]= {"TLS_AES_128_GCM_SHA256", "TLS_AES_256_GCM_SHA384", "TLS_CHACHA20_POLY1305_SHA256"};
+  my_bool verify= 0;
+
+  SKIP_MAXSCALE;
+
+  for (i=0; i < 3; i++)
+  {
+    const char *tls_version;
+    mysql= mysql_init(NULL);
+
+    mysql_ssl_set(mysql, NULL, NULL, NULL, NULL, NULL);
+    mysql_optionsv(mysql, MYSQL_OPT_SSL_VERIFY_SERVER_CERT, &verify);
+    mysql_optionsv(mysql, MYSQL_OPT_SSL_CIPHER, ciphers[i]);
+
+    if (!my_test_connect(mysql, hostname, username,
+                         password, schema, port, socketname, 0))
+    {
+      diag("error: %s", mysql_error(mysql));
+      return FAIL;
+    }
+
+    FAIL_IF(strcmp(ciphers[i], mysql_get_ssl_cipher(mysql)), "Cipher mismatch");
+    mariadb_get_infov(mysql, MARIADB_CONNECTION_TLS_VERSION, &tls_version);
+    FAIL_IF(strcmp(tls_version, "TLSv1.3"), "TLS version mismatch");
+
+    mysql_close(mysql);
+  }
+  return OK;
+}
+#endif
+
+static int test_conc589(MYSQL *my)
+{
+  MYSQL *mysql= mysql_init(NULL);
+  int rc;
+  my_bool reconnect= 1, verify= 0;
+  unsigned long last_thread_id= 0;
+
+  SKIP_MAXSCALE;
+
+  mysql_options(mysql, MYSQL_OPT_RECONNECT, &reconnect);
+  mysql_options(mysql, MYSQL_OPT_SSL_VERIFY_SERVER_CERT, &verify);
+
+  if (!my_test_connect(mysql, hostname, username,
+                       password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
+  {
+    diag("error: %s", mysql_error(mysql));
+    return FAIL;
+  }
+
+  rc= mysql_query(mysql, "SET SESSION wait_timeout=5");
+  check_mysql_rc(rc, mysql);
+
+  last_thread_id= mysql_thread_id(mysql);
+  rc= mysql_query(mysql, "SET @a:=1");
+  check_mysql_rc(rc, mysql);
+
+  sleep(10);
+
+  rc= mysql_query(mysql, "SET @a:=2");
+  check_mysql_rc(rc, mysql);
+  FAIL_IF(mysql_thread_id(mysql) == last_thread_id, "Expected new connection id");
+  last_thread_id= mysql_thread_id(mysql);
+
+  mysql_kill(my, last_thread_id);
+
+  sleep(10);
+
+  rc= mysql_query(mysql, "SET @a:=3");
+  check_mysql_rc(rc, mysql);
+  FAIL_IF(mysql_thread_id(mysql) == last_thread_id, "Expected new connection id");
+  mysql_close(mysql);
+  return OK;
+}
+
+#ifdef WIN32
+static int test_conc760(MYSQL *my)
+{
+  MYSQL *mysql= mysql_init(NULL);
+  MYSQL_RES *result;
+  MYSQL_ROW row;
+  int rc;
+  char named_pipe_name[128];
+  my_bool reconnect= 1, verify= 0;
+  unsigned long last_thread_id= 0;
+  unsigned int protocol= MYSQL_PROTOCOL_PIPE;
+  my_bool have_named_pipe= 0;
+
+  SKIP_MAXSCALE;
+
+  rc= mysql_query(my, "select @@named_pipe, @@socket");
+  check_mysql_rc(rc, mysql);
+
+  if ((result= mysql_store_result(my)))
+  {
+    if((row= mysql_fetch_row(result)))
+      have_named_pipe= atoi(row[0]);
+    strncpy(named_pipe_name, row[1], sizeof(named_pipe_name)-1);
+    named_pipe_name[sizeof(named_pipe_name)-1]= '\0';
+    mysql_free_result(result);
+  }
+
+  if (!have_named_pipe)
+  {
+    diag("Server doesn't support named pipes");
+    return SKIP;
+  }
+
+  mysql_options(mysql, MYSQL_OPT_RECONNECT, &reconnect);
+  mysql_options(mysql, MYSQL_OPT_SSL_VERIFY_SERVER_CERT, &verify);
+  mysql_options(mysql, MYSQL_OPT_PROTOCOL, &protocol);
+
+  if (!my_test_connect(mysql, hostname, username,
+                       password, schema, port, named_pipe_name, CLIENT_REMEMBER_OPTIONS))
+  {
+    diag("error: %s", mysql_error(mysql));
+    return FAIL;
+  }
+
+  rc= mysql_query(mysql, "SET SESSION wait_timeout=5");
+  check_mysql_rc(rc, mysql);
+
+  last_thread_id= mysql_thread_id(mysql);
+  if ((rc= mysql_query(mysql, "SELECT 1")) || (result= mysql_store_result(mysql)) == NULL)
+    check_mysql_rc(rc, mysql);
+
+  mysql_free_result(result);
+  sleep(10);
+
+  rc= mysql_query(mysql, "SELECT 2");
+  check_mysql_rc(rc, mysql);
+  if (result= mysql_store_result(mysql))
+    mysql_free_result(result);
+  FAIL_IF(mysql_thread_id(mysql) == last_thread_id, "Expected new connection id");
+  last_thread_id= mysql_thread_id(mysql);
+
+  mysql_kill(my, last_thread_id);
+
+  sleep(10);
+
+  if ((rc= mysql_query(mysql, "SELECT 3")) || (result= mysql_store_result(mysql)) == NULL)
+    check_mysql_rc(rc, mysql);
+  mysql_free_result(result);
+  FAIL_IF(mysql_thread_id(mysql) == last_thread_id, "Expected new connection id");
+  mysql_close(mysql);
+  return OK;
+}
+#endif
+
 struct my_tests_st my_tests[] = {
+#ifdef WIN32
+  {"test_conc760", test_conc760, TEST_CONNECTION_DEFAULT, 0, NULL, NULL},
+#endif
+  {"test_conc589", test_conc589, TEST_CONNECTION_DEFAULT, 0, NULL, NULL},
+#ifdef HAVE_test_conc748
+  {"test_conc748", test_conc748, TEST_CONNECTION_NONE, 0, NULL, NULL},
+#endif
+  {"test_conc505", test_conc505, TEST_CONNECTION_NONE, 0, NULL, NULL},
+  {"test_conc632", test_conc632, TEST_CONNECTION_NONE, 0, NULL, NULL},
+  {"test_status_callback", test_status_callback, TEST_CONNECTION_NONE, 0, NULL, NULL},
+  {"test_conc365", test_conc365, TEST_CONNECTION_NONE, 0, NULL, NULL},
+  {"test_conc365_reconnect", test_conc365_reconnect, TEST_CONNECTION_DEFAULT, 0, NULL, NULL},
+  {"test_conn_str", test_conn_str, TEST_CONNECTION_NONE, 0, NULL, NULL},
+  {"test_conn_str_1", test_conn_str_1, TEST_CONNECTION_NONE, 0, NULL, NULL},
+  {"test_conc544", test_conc544, TEST_CONNECTION_DEFAULT, 0, NULL, NULL},
   {"test_conc490", test_conc490, TEST_CONNECTION_NONE, 0, NULL, NULL},
   {"test_gtid", test_gtid, TEST_CONNECTION_DEFAULT, 0, NULL, NULL},
   {"test_conc496", test_conc496, TEST_CONNECTION_DEFAULT, 0, NULL, NULL},
