@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 /* my_setwd() and my_getwd() works with intern_filenames !! */
 
@@ -22,7 +22,7 @@
 #ifdef HAVE_GETWD
 #include <sys/param.h>
 #endif
-#if defined(__WIN__)
+#if defined(_WIN32)
 #include <m_ctype.h>
 #include <dos.h>
 #include <direct.h>
