@@ -37,8 +37,9 @@ static const uint FOR_WRITING= 1;
 class Url_http: public Url {
   protected:
   const LEX_STRING host, port, path;
-  bool ssl;
   LEX_STRING proxy_host, proxy_port;
+  my_socket fd;
+  bool ssl;
 
   bool use_proxy()
   {
@@ -47,11 +48,12 @@ class Url_http: public Url {
 
   Url_http(LEX_STRING &url_arg, LEX_STRING &host_arg,
           LEX_STRING &port_arg, LEX_STRING &path_arg, bool ssl_arg) :
-    Url(url_arg), host(host_arg), port(port_arg), path(path_arg), ssl(ssl_arg)
+    Url(url_arg), host(host_arg), port(port_arg), path(path_arg),
+    fd(INVALID_SOCKET), ssl(ssl_arg)
     {
       proxy_host.length= 0;
     }
-  ~Url_http()
+  ~Url_http() override
   {
     my_free(host.str);
     my_free(port.str);
@@ -60,8 +62,9 @@ class Url_http: public Url {
   }
 
   public:
-  int send(const char* data, size_t data_length);
-  int set_proxy(const char *proxy, size_t proxy_len)
+  void abort() override;
+  int send(const char* data, size_t data_length) override;
+  int set_proxy(const char *proxy, size_t proxy_len) override
   {
     if (use_proxy())
     {
@@ -143,9 +146,9 @@ Url* http_create(const char *url, size_t url_length)
   if (!host.length || !port.length || path.str[0] != '/')
     return NULL;
 
-  host.str= my_strndup(host.str, host.length, MYF(MY_WME));
-  port.str= my_strndup(port.str, port.length, MYF(MY_WME));
-  path.str= my_strndup(path.str, path.length, MYF(MY_WME));
+  host.str= my_strndup(PSI_INSTRUMENT_ME, host.str, host.length, MYF(MY_WME));
+  port.str= my_strndup(PSI_INSTRUMENT_ME, port.str, port.length, MYF(MY_WME));
+  path.str= my_strndup(PSI_INSTRUMENT_ME, path.str, path.length, MYF(MY_WME));
 
   if (!host.str || !port.str || !path.str)
   {
@@ -158,13 +161,18 @@ Url* http_create(const char *url, size_t url_length)
   return new Url_http(full_url, host, port, path, ssl);
 }
 
+void Url_http::abort()
+{
+  if (fd != INVALID_SOCKET)
+    closesocket(fd); // interrupt I/O waits
+}
+
 /* do the vio_write and check that all data were sent ok */
 #define write_check(VIO, DATA, LEN)             \
   (vio_write((VIO), (uchar*)(DATA), (LEN)) != (LEN))
 
 int Url_http::send(const char* data, size_t data_length)
 {
-  my_socket fd= INVALID_SOCKET;
   char buf[1024];
   size_t len= 0;
 
@@ -180,6 +188,7 @@ int Url_http::send(const char* data, size_t data_length)
     return 1;
   }
 
+  DBUG_ASSERT(fd == INVALID_SOCKET);
   for (addr= addrs; addr != NULL; addr= addr->ai_next)
   {
     fd= socket(addr->ai_family, addr->ai_socktype, addr->ai_protocol);
@@ -208,6 +217,7 @@ int Url_http::send(const char* data, size_t data_length)
     sql_print_error("feedback plugin: vio_new failed for url '%s'",
                     full_url.str);
     closesocket(fd);
+    fd= INVALID_SOCKET;
     return 1;
   }
 
@@ -236,6 +246,7 @@ int Url_http::send(const char* data, size_t data_length)
         free_vio_ssl_acceptor_fd(ssl_fd);
       closesocket(fd);
       vio_delete(vio);
+      fd= INVALID_SOCKET;
       return 1;
     }
   }
@@ -334,6 +345,7 @@ int Url_http::send(const char* data, size_t data_length)
   }
 #endif
 
+  fd= INVALID_SOCKET;
   return res;
 }
 
