@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+#!/usr/bin/env perl
 # Test of table elimination feature
 
 use Cwd;
@@ -34,18 +34,7 @@ $select_current_full_facts="
                                A2.fromdate=(select MAX(fromdate) from
                                             elim_attr2 where id=A2.id);
 ";
-$select_current_full_facts="
-  select 
-    F.id, A1.attr1, A2.attr2
-  from 
-    elim_facts F 
-    left join elim_attr1 A1 on A1.id=F.id
-    left join elim_attr2 A2 on A2.id=F.id and 
-                               A2.fromdate=(select MAX(fromdate) from
-                                            elim_attr2 where id=F.id);
-";
-# TODO: same as above but for some given date also? 
-# TODO: 
+# TODO: same as above but for some given date also?
 
 
 ####
@@ -62,6 +51,7 @@ $start_time=new Benchmark;
 goto select_test if ($opt_skip_create);
 
 print "Creating tables\n";
+$dbh->do("drop view elim_current_facts");
 $dbh->do("drop table elim_facts" . $server->{'drop_attr'});
 $dbh->do("drop table elim_attr1" . $server->{'drop_attr'});
 $dbh->do("drop table elim_attr2" . $server->{'drop_attr'});
@@ -76,7 +66,7 @@ do_many($dbh,$server->create("elim_attr1",
 			     ["id integer",
                               "attr1 integer"],
 			     ["primary key (id)",
-                              "key (attr1)"]));
+                              "index ix_attr1 (attr1)"]));
 
 # Attribute2, time-versioned
 do_many($dbh,$server->create("elim_attr2",
@@ -84,7 +74,7 @@ do_many($dbh,$server->create("elim_attr2",
                               "attr2 integer",
                               "fromdate date"],
 			     ["primary key (id, fromdate)",
-                              "key (attr2,fromdate)"]));
+                              "index ix_attr2 (attr2,fromdate)"]));
 
 #NOTE: ignoring: if ($limits->{'views'})
 $dbh->do("drop view elim_current_facts");
@@ -304,8 +294,8 @@ if ($opt_lock_tables)
 }
 if (!$opt_skip_delete)
 {
-  do_query($dbh,"drop table elim_facts, elim_attr1, elim_attr2" . $server->{'drop_attr'});
   $dbh->do("drop view elim_current_facts");
+  do_query($dbh,"drop table elim_facts, elim_attr1, elim_attr2" . $server->{'drop_attr'});
 }
 
 if ($opt_fast && defined($server->{vacuum}))
