@@ -381,6 +381,8 @@ int ha_json_table::rnd_init(bool scan)
 
 static void store_json_in_field(Field *f, const json_engine_t *je)
 {
+  String res_tmp("", 0, je->s.cs);
+
   switch (je->value_type)
   {
   case JSON_VALUE_NULL:
@@ -401,7 +403,8 @@ static void store_json_in_field(Field *f, const json_engine_t *je)
   default:
     break;
   };
-  f->store((const char *) je->value, (uint32) je->value_len, je->s.cs);
+  st_append_json(&res_tmp, je->s.cs, je->value, je->value_len);
+  f->store((const char *) res_tmp.ptr(), (uint32) res_tmp.length(), je->s.cs);
 }
 
 
@@ -1046,8 +1049,7 @@ int Json_table_column::On_response::respond(Json_table_column *jc, Field *f,
       return 1;
     case Json_table_column::RESPONSE_DEFAULT:
       f->set_notnull();
-      f->store(m_default.str,
-          m_default.length, jc->m_defaults_cs);
+      m_default->save_in_field(f, TRUE);
       break;
   }
   return 0;
@@ -1057,7 +1059,11 @@ int Json_table_column::On_response::respond(Json_table_column *jc, Field *f,
 int Json_table_column::On_response::print(const char *name, String *str) const
 {
   LEX_CSTRING resp;
-  const LEX_CSTRING *ds= NULL;
+
+  char valbuf[512];
+  String val(valbuf, sizeof(valbuf), str->charset());
+  String *ds= NULL;
+
   if (m_response == Json_table_column::RESPONSE_NOT_SPECIFIED)
     return 0;
 
@@ -1072,7 +1078,7 @@ int Json_table_column::On_response::print(const char *name, String *str) const
     case Json_table_column::RESPONSE_DEFAULT:
     {
       lex_string_set3(&resp, STRING_WITH_LEN("DEFAULT"));
-      ds= &m_default;
+      ds= m_default->val_str(&val);
       break;
     }
     default:
@@ -1081,9 +1087,14 @@ int Json_table_column::On_response::print(const char *name, String *str) const
   }
 
   return (str->append(' ') || str->append(resp)  ||
-          (ds && (str->append(STRING_WITH_LEN(" '")) ||
-                  str->append_for_single_quote(ds->str, ds->length) ||
-                  str->append('\''))) ||
+          (ds &&
+            (str->append(' ') ||
+             (m_default->result_type()==STRING_RESULT && str->append('\''))||
+
+             str->append_for_single_quote(ds) ||
+
+             (m_default->result_type()==STRING_RESULT && str->append('\''))))||
+
           str->append(STRING_WITH_LEN(" ON ")) ||
           str->append(name, strlen(name)));
 }
@@ -1162,7 +1173,7 @@ bool push_table_function_arg_context(LEX *lex, MEM_ROOT *alloc)
 bool Table_function_json_table::setup(THD *thd, TABLE_LIST *sql_table,
                                      SELECT_LEX *s_lex)
 {
-  thd->where= "JSON_TABLE argument";
+  thd->where= THD_WHERE::JSON_TABLE_ARGUMENT;
 
   if (!m_context_setup_done)
   {
