@@ -1,5 +1,5 @@
 /* Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
-   Copyright (c) 2009, 2020, MariaDB Corporation.
+   Copyright (c) 2009, 2021, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -120,11 +120,15 @@ bool trans_begin(THD *thd, uint flags)
   if (thd->in_multi_stmt_transaction_mode() ||
       (thd->variables.option_bits & OPTION_TABLE_LOCK))
   {
+    bool was_in_trans= thd->server_status &
+      (SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
     thd->variables.option_bits&= ~OPTION_TABLE_LOCK;
     thd->server_status&=
       ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
     DBUG_PRINT("info", ("clearing SERVER_STATUS_IN_TRANS"));
     res= MY_TEST(ha_commit_trans(thd, TRUE));
+    if (was_in_trans)
+      trans_reset_one_shot_chistics(thd);
 #ifdef WITH_WSREP
     if (wsrep_thd_is_local(thd))
     {
@@ -133,7 +137,7 @@ bool trans_begin(THD *thd, uint flags)
 #endif /* WITH_WSREP */
   }
 
-  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_KEEP_LOG);
+  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_BINLOG_THIS_TRX);
 
   /*
     The following set should not be needed as transaction state should
@@ -151,7 +155,7 @@ bool trans_begin(THD *thd, uint flags)
     Release transactional metadata locks only after the
     transaction has been committed.
   */
-  thd->mdl_context.release_transactional_locks();
+  thd->release_transactional_locks();
 
   // The RO/RW options are mutually exclusive.
   DBUG_ASSERT(!((flags & MYSQL_START_TRANS_OPT_READ_ONLY) &&
@@ -191,7 +195,7 @@ bool trans_begin(THD *thd, uint flags)
   }
 
 #ifdef WITH_WSREP
-  if (wsrep_thd_is_local(thd))
+  if (WSREP(thd) && wsrep_thd_is_local(thd))
   {
     if (wsrep_sync_wait(thd))
       DBUG_RETURN(TRUE);
@@ -255,11 +259,15 @@ bool trans_begin(THD *thd, uint flags)
 bool trans_commit(THD *thd)
 {
   int res;
+  PSI_stage_info org_stage;
   DBUG_ENTER("trans_commit");
 
   if (trans_check(thd))
     DBUG_RETURN(TRUE);
 
+  thd->backup_stage(&org_stage);
+  THD_STAGE_INFO(thd, stage_commit);
+
   thd->server_status&=
     ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
   DBUG_PRINT("info", ("clearing SERVER_STATUS_IN_TRANS"));
@@ -280,7 +288,7 @@ bool trans_commit(THD *thd)
   else
     repl_semisync_master.wait_after_commit(thd, FALSE);
 #endif
-  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_KEEP_LOG);
+  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_BINLOG_THIS_TRX);
   thd->transaction->all.reset();
   thd->lex->start_transaction_opt= 0;
 
@@ -288,6 +296,7 @@ bool trans_commit(THD *thd)
   DBUG_ASSERT(thd->m_transaction_psi == NULL);
   trans_track_end_trx(thd);
 
+  THD_STAGE_INFO(thd, org_stage);
   DBUG_RETURN(MY_TEST(res));
 }
 
@@ -320,6 +329,10 @@ bool trans_commit_implicit(THD *thd)
   if (thd->in_multi_stmt_transaction_mode() ||
       (thd->variables.option_bits & OPTION_TABLE_LOCK))
   {
+    PSI_stage_info org_stage;
+    thd->backup_stage(&org_stage);
+    THD_STAGE_INFO(thd, stage_commit_implicit);
+
     /* Safety if one did "drop table" on locked tables */
     if (!thd->locked_tables_mode)
       thd->variables.option_bits&= ~OPTION_TABLE_LOCK;
@@ -327,9 +340,11 @@ bool trans_commit_implicit(THD *thd)
       ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
     DBUG_PRINT("info", ("clearing SERVER_STATUS_IN_TRANS"));
     res= MY_TEST(ha_commit_trans(thd, TRUE));
+
+    THD_STAGE_INFO(thd, org_stage);
   }
 
-  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_KEEP_LOG);
+  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_BINLOG_THIS_TRX);
   thd->transaction->all.reset();
 
   /* The transaction should be marked as complete in P_S. */
@@ -361,11 +376,15 @@ bool trans_commit_implicit(THD *thd)
 bool trans_rollback(THD *thd)
 {
   int res;
+  PSI_stage_info org_stage;
   DBUG_ENTER("trans_rollback");
 
   if (trans_check(thd))
     DBUG_RETURN(TRUE);
 
+  thd->backup_stage(&org_stage);
+  THD_STAGE_INFO(thd, stage_rollback);
+
   thd->server_status&=
     ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
   DBUG_PRINT("info", ("clearing SERVER_STATUS_IN_TRANS"));
@@ -374,7 +393,7 @@ bool trans_rollback(THD *thd)
   repl_semisync_master.wait_after_rollback(thd, FALSE);
 #endif
   /* Reset the binlog transaction marker */
-  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_KEEP_LOG |
+  thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_BINLOG_THIS_TRX |
                                  OPTION_GTID_BEGIN);
   thd->transaction->all.reset();
   thd->lex->start_transaction_opt= 0;
@@ -384,6 +403,7 @@ bool trans_rollback(THD *thd)
 
   trans_track_end_trx(thd);
 
+  THD_STAGE_INFO(thd, org_stage);
   DBUG_RETURN(MY_TEST(res));
 }
 
@@ -406,8 +426,12 @@ bool trans_rollback(THD *thd)
 bool trans_rollback_implicit(THD *thd)
 {
   int res;
+  PSI_stage_info org_stage;
   DBUG_ENTER("trans_rollback_implict");
 
+  thd->backup_stage(&org_stage);
+  THD_STAGE_INFO(thd, stage_rollback_implicit);
+
   /*
     Always commit/rollback statement transaction before manipulating
     with the normal one.
@@ -416,7 +440,7 @@ bool trans_rollback_implicit(THD *thd)
   */
   DBUG_ASSERT(thd->transaction->stmt.is_empty() && !thd->in_sub_stmt);
 
-  thd->server_status&= ~SERVER_STATUS_IN_TRANS;
+  thd->server_status&= ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
   DBUG_PRINT("info", ("clearing SERVER_STATUS_IN_TRANS"));
   res= ha_rollback_trans(thd, true);
   /*
@@ -424,7 +448,7 @@ bool trans_rollback_implicit(THD *thd)
     of new transacton in @@autocommit=1 mode. This is necessary to
     preserve backward compatibility.
   */
-  thd->variables.option_bits&= ~(OPTION_KEEP_LOG);
+  thd->variables.option_bits&= ~(OPTION_BINLOG_THIS_TRX);
   thd->transaction->all.reset();
 
   /* Rollback should clear transaction_rollback_request flag. */
@@ -434,6 +458,7 @@ bool trans_rollback_implicit(THD *thd)
 
   trans_track_end_trx(thd);
 
+  THD_STAGE_INFO(thd, org_stage);
   DBUG_RETURN(MY_TEST(res));
 }
 
@@ -463,17 +488,23 @@ bool trans_commit_stmt(THD *thd)
     a savepoint for each nested statement, and release the
     savepoint when statement has succeeded.
   */
-  DBUG_ASSERT(! thd->in_sub_stmt);
+  DBUG_ASSERT(!(thd->in_sub_stmt));
 
   thd->merge_unsafe_rollback_flags();
 
   if (thd->transaction->stmt.ha_list)
   {
+    PSI_stage_info org_stage;
+    thd->backup_stage(&org_stage);
+    THD_STAGE_INFO(thd, stage_commit);
+
     res= ha_commit_trans(thd, FALSE);
     if (! thd->in_active_multi_stmt_transaction())
     {
       trans_reset_one_shot_chistics(thd);
     }
+
+    THD_STAGE_INFO(thd, org_stage);
   }
 
   mysql_mutex_assert_not_owner(&LOCK_prepare_ordered);
@@ -532,9 +563,15 @@ bool trans_rollback_stmt(THD *thd)
 
   if (thd->transaction->stmt.ha_list)
   {
+    PSI_stage_info org_stage;
+    thd->backup_stage(&org_stage);
+    THD_STAGE_INFO(thd, stage_rollback);
+
     ha_rollback_trans(thd, FALSE);
     if (! thd->in_active_multi_stmt_transaction())
       trans_reset_one_shot_chistics(thd);
+
+    THD_STAGE_INFO(thd, org_stage);
   }
 
 #ifdef HAVE_REPLICATION
@@ -667,36 +704,9 @@ bool trans_rollback_to_savepoint(THD *thd, LEX_CSTRING name)
   if (thd->transaction->xid_state.check_has_uncommitted_xa())
     DBUG_RETURN(TRUE);
 
-  /**
-    Checking whether it is safe to release metadata locks acquired after
-    savepoint, if rollback to savepoint is successful.
-  
-    Whether it is safe to release MDL after rollback to savepoint depends
-    on storage engines participating in transaction:
-  
-    - InnoDB doesn't release any row-locks on rollback to savepoint so it
-      is probably a bad idea to release MDL as well.
-    - Binary log implementation in some cases (e.g when non-transactional
-      tables involved) may choose not to remove events added after savepoint
-      from transactional cache, but instead will write them to binary
-      log accompanied with ROLLBACK TO SAVEPOINT statement. Since the real
-      write happens at the end of transaction releasing MDL on tables
-      mentioned in these events (i.e. acquired after savepoint and before
-      rollback ot it) can break replication, as concurrent DROP TABLES
-      statements will be able to drop these tables before events will get
-      into binary log,
-  
-    For backward-compatibility reasons we always release MDL if binary
-    logging is off.
-  */
-  bool mdl_can_safely_rollback_to_savepoint=
-                (!((WSREP_EMULATE_BINLOG_NNULL(thd) || mysql_bin_log.is_open())
-                 && thd->variables.sql_log_bin) ||
-                 ha_rollback_to_savepoint_can_release_mdl(thd));
-
   if (ha_rollback_to_savepoint(thd, sv))
     res= TRUE;
-  else if (((thd->variables.option_bits & OPTION_KEEP_LOG) ||
+  else if (((thd->variables.option_bits & OPTION_BINLOG_THIS_TRX) ||
             thd->transaction->all.modified_non_trans_table) &&
            !thd->slave_thread)
     push_warning(thd, Sql_condition::WARN_LEVEL_WARN,
@@ -705,7 +715,15 @@ bool trans_rollback_to_savepoint(THD *thd, LEX_CSTRING name)
 
   thd->transaction->savepoints= sv;
 
-  if (!res && mdl_can_safely_rollback_to_savepoint)
+  if (res)
+    /* An error occurred during rollback; we cannot release any MDL */;
+  else if (thd->variables.sql_log_bin &&
+           (WSREP_EMULATE_BINLOG_NNULL(thd) || mysql_bin_log.is_open()))
+    /* In some cases (such as with non-transactional tables) we may
+    choose to preserve events that were added after the SAVEPOINT,
+    delimiting them by SAVEPOINT and ROLLBACK TO SAVEPOINT statements.
+    Prematurely releasing MDL on such objects would break replication. */;
+  else if (ha_rollback_to_savepoint_can_release_mdl(thd))
     thd->mdl_context.rollback_to_savepoint(sv->mdl_savepoint);
 
   DBUG_RETURN(MY_TEST(res));
