@@ -126,10 +126,11 @@ public:
     }
     return true;
   }
-  static void lf_hash_initializer(LF_HASH *hash __attribute__((unused)),
-                                  XID_cache_element *element,
-                                  XID_cache_insert_element *new_element)
+  static void lf_hash_initializer(LF_HASH *, void *el, const void *ie)
   {
+    XID_cache_element *element= static_cast<XID_cache_element*>(el);
+    XID_cache_insert_element *new_element=
+      static_cast<XID_cache_insert_element*>(const_cast<void*>(ie));
     DBUG_ASSERT(!element->is_set(ACQUIRED | RECOVERED));
     element->rm_error= 0;
     element->xa_state= new_element->xa_state;
@@ -146,11 +147,11 @@ public:
     DBUG_ASSERT(!reinterpret_cast<XID_cache_element*>(ptr + LF_HASH_OVERHEAD)
 		->is_set(ACQUIRED));
   }
-  static uchar *key(const XID_cache_element *element, size_t *length,
-                    my_bool not_used __attribute__((unused)))
+  static const uchar *key(const void *el, size_t *length, my_bool)
   {
-    *length= element->xid.key_length();
-    return element->xid.key();
+    const XID &xid= reinterpret_cast<const XID_cache_element*>(el)->xid;
+    *length= xid.key_length();
+    return xid.key();
   }
 };
 
@@ -179,6 +180,13 @@ void XID_STATE::set_error(uint error)
     xid_cache_element->rm_error= error;
 }
 
+void XID_STATE::set_rollback_only()
+{
+  xid_cache_element->xa_state= XA_ROLLBACK_ONLY;
+  if (current_thd)
+    MYSQL_SET_TRANSACTION_XA_STATE(current_thd->m_transaction_psi,
+                                   XA_ROLLBACK_ONLY);
+}
 
 void XID_STATE::er_xaer_rmfail() const
 {
@@ -221,11 +229,10 @@ void xid_cache_init()
 {
   xid_cache_inited= true;
   lf_hash_init(&xid_cache, sizeof(XID_cache_element), LF_HASH_UNIQUE, 0, 0,
-               (my_hash_get_key) XID_cache_element::key, &my_charset_bin);
+               XID_cache_element::key, &my_charset_bin);
   xid_cache.alloc.constructor= XID_cache_element::lf_alloc_constructor;
   xid_cache.alloc.destructor= XID_cache_element::lf_alloc_destructor;
-  xid_cache.initializer=
-    (lf_hash_initializer) XID_cache_element::lf_hash_initializer;
+  xid_cache.initializer= XID_cache_element::lf_hash_initializer;
 }
 
 
@@ -331,9 +338,10 @@ struct xid_cache_iterate_arg
   void *argument;
 };
 
-static my_bool xid_cache_iterate_callback(XID_cache_element *element,
-                                          xid_cache_iterate_arg *arg)
+static my_bool xid_cache_iterate_callback(void *el, void *a)
 {
+  XID_cache_element *element= static_cast<XID_cache_element*>(el);
+  xid_cache_iterate_arg *arg= static_cast<xid_cache_iterate_arg*>(a);
   my_bool res= FALSE;
   if (element->lock())
   {
@@ -348,8 +356,7 @@ static int xid_cache_iterate(THD *thd, my_hash_walk_action action, void *arg)
   xid_cache_iterate_arg argument= { action, arg };
   return thd->fix_xid_hash_pins() ? -1 :
          lf_hash_iterate(&xid_cache, thd->xid_hash_pins,
-                         (my_hash_walk_action) xid_cache_iterate_callback,
-                         &argument);
+                         xid_cache_iterate_callback, &argument);
 }
 
 
@@ -500,6 +507,40 @@ bool trans_xa_end(THD *thd)
 }
 
 
+/*
+  Get the BACKUP_COMMIT lock for the duration of the XA.
+
+  The metadata lock which will ensure that COMMIT is blocked
+   by active FLUSH TABLES WITH READ LOCK (and vice versa COMMIT in
+   progress blocks FTWRL) and also by MDL_BACKUP_WAIT_COMMIT.
+   We allow FLUSHer to COMMIT; we assume FLUSHer knows what it does.
+
+   Note that the function sets thd->backup_lock on sucess. The caller needs
+   to reset thd->backup_commit_lock before returning!
+*/
+
+static bool trans_xa_get_backup_lock(THD *thd, MDL_request *mdl_request)
+{
+  DBUG_ASSERT(thd->backup_commit_lock == 0);
+  MDL_REQUEST_INIT(mdl_request, MDL_key::BACKUP, "", "", MDL_BACKUP_COMMIT,
+                   MDL_EXPLICIT);
+  if (thd->mdl_context.acquire_lock(mdl_request,
+                                    thd->variables.lock_wait_timeout))
+    return 1;
+  thd->backup_commit_lock= mdl_request;
+  return 0;
+}
+
+static inline void trans_xa_release_backup_lock(THD *thd)
+{
+  if (thd->backup_commit_lock)
+  {
+    thd->mdl_context.release_lock(thd->backup_commit_lock->ticket);
+    thd->backup_commit_lock= 0;
+  }
+}
+
+
 /**
   Put a XA transaction in the PREPARED state.
 
@@ -522,21 +563,12 @@ bool trans_xa_prepare(THD *thd)
     my_error(ER_XAER_NOTA, MYF(0));
   else
   {
-    /*
-      Acquire metadata lock which will ensure that COMMIT is blocked
-      by active FLUSH TABLES WITH READ LOCK (and vice versa COMMIT in
-      progress blocks FTWRL).
-
-      We allow FLUSHer to COMMIT; we assume FLUSHer knows what it does.
-    */
     MDL_request mdl_request;
-    MDL_REQUEST_INIT(&mdl_request, MDL_key::BACKUP, "", "", MDL_BACKUP_COMMIT,
-                     MDL_STATEMENT);
-    if (thd->mdl_context.acquire_lock(&mdl_request,
-                                      thd->variables.lock_wait_timeout) ||
+    if (trans_xa_get_backup_lock(thd, &mdl_request) ||
         ha_prepare(thd))
     {
       if (!mdl_request.ticket)
+        /* Failed to get the backup lock */
         ha_rollback_trans(thd, TRUE);
       thd->variables.option_bits&= ~(OPTION_BEGIN | OPTION_BINLOG_THIS_TRX);
       thd->transaction->all.reset();
@@ -547,11 +579,25 @@ bool trans_xa_prepare(THD *thd)
     }
     else
     {
-      thd->transaction->xid_state.xid_cache_element->xa_state= XA_PREPARED;
-      MYSQL_SET_TRANSACTION_XA_STATE(thd->m_transaction_psi, XA_PREPARED);
+      if (thd->transaction->xid_state.xid_cache_element->xa_state !=
+          XA_ROLLBACK_ONLY)
+      {
+        thd->transaction->xid_state.xid_cache_element->xa_state= XA_PREPARED;
+        MYSQL_SET_TRANSACTION_XA_STATE(thd->m_transaction_psi, XA_PREPARED);
+      }
+      else
+      {
+        /*
+          In the non-err case, XA_ROLLBACK_ONLY should only be set by a slave
+          thread which prepared an empty transaction, to prevent binlogging a
+          standalone XA COMMIT.
+        */
+        DBUG_ASSERT(thd->rgi_slave && !(thd->transaction->all.ha_list));
+      }
       res= thd->variables.pseudo_slave_mode || thd->slave_thread ?
         slave_applier_reset_xa_trans(thd) : 0;
     }
+    trans_xa_release_backup_lock(thd);
   }
 
   DBUG_RETURN(res);
@@ -615,19 +661,8 @@ bool trans_xa_commit(THD *thd)
         res= 1;
         goto _end_external_xid;
       }
-
       res= xa_trans_rolled_back(xs);
-      /*
-        Acquire metadata lock which will ensure that COMMIT is blocked
-        by active FLUSH TABLES WITH READ LOCK (and vice versa COMMIT in
-        progress blocks FTWRL).
-
-        We allow FLUSHer to COMMIT; we assume FLUSHer knows what it does.
-      */
-      MDL_REQUEST_INIT(&mdl_request, MDL_key::BACKUP, "", "", MDL_BACKUP_COMMIT,
-                       MDL_EXPLICIT);
-      if (thd->mdl_context.acquire_lock(&mdl_request,
-                                        thd->variables.lock_wait_timeout))
+      if (trans_xa_get_backup_lock(thd, &mdl_request))
       {
         /*
           We can't rollback an XA transaction on lock failure due to
@@ -639,14 +674,11 @@ bool trans_xa_commit(THD *thd)
         res= true;
         goto _end_external_xid;
       }
-      else
-      {
-        thd->backup_commit_lock= &mdl_request;
-      }
       DBUG_ASSERT(!xid_state.xid_cache_element);
 
       xid_state.xid_cache_element= xs;
       ha_commit_or_rollback_by_xid(thd->lex->xid, !res);
+
       if (!res && thd->is_error())
       {
         // hton completion error retains xs/xid in the cache,
@@ -662,11 +694,7 @@ bool trans_xa_commit(THD *thd)
       res= res || thd->is_error();
       if (!xid_deleted)
         xs->acquired_to_recovered();
-      if (mdl_request.ticket)
-      {
-        thd->mdl_context.release_lock(mdl_request.ticket);
-        thd->backup_commit_lock= 0;
-      }
+      trans_xa_release_backup_lock(thd);
     }
     else
       my_error(ER_XAER_NOTA, MYF(0));
@@ -689,7 +717,8 @@ bool trans_xa_commit(THD *thd)
     if ((res= MY_TEST(r)))
       my_error(r == 1 ? ER_XA_RBROLLBACK : ER_XAER_RMERR, MYF(0));
   }
-  else if (thd->transaction->xid_state.xid_cache_element->xa_state == XA_PREPARED)
+  else if (thd->transaction->xid_state.xid_cache_element->xa_state ==
+           XA_PREPARED)
   {
     MDL_request mdl_request;
     if (thd->lex->xa_opt != XA_NONE)
@@ -698,18 +727,7 @@ bool trans_xa_commit(THD *thd)
       DBUG_RETURN(TRUE);
     }
 
-    /*
-      Acquire metadata lock which will ensure that COMMIT is blocked
-      by active FLUSH TABLES WITH READ LOCK (and vice versa COMMIT in
-      progress blocks FTWRL).
-
-      We allow FLUSHer to COMMIT; we assume FLUSHer knows what it does.
-    */
-    MDL_REQUEST_INIT(&mdl_request, MDL_key::BACKUP, "", "", MDL_BACKUP_COMMIT,
-                     MDL_TRANSACTION);
-
-    if (thd->mdl_context.acquire_lock(&mdl_request,
-                                      thd->variables.lock_wait_timeout))
+    if (trans_xa_get_backup_lock(thd, &mdl_request))
     {
       /*
         We can't rollback an XA transaction on lock failure due to
@@ -736,6 +754,7 @@ bool trans_xa_commit(THD *thd)
       }
 
       thd->m_transaction_psi= NULL;
+      trans_xa_release_backup_lock(thd);
     }
   }
   else
@@ -773,7 +792,8 @@ bool trans_xa_commit(THD *thd)
 bool trans_xa_rollback(THD *thd)
 {
   XID_STATE &xid_state= thd->transaction->xid_state;
-
+  MDL_request mdl_request;
+  bool error;
   DBUG_ENTER("trans_xa_rollback");
 
   if (!xid_state.is_explicit_XA() ||
@@ -794,7 +814,6 @@ bool trans_xa_rollback(THD *thd)
     {
       bool res;
       bool xid_deleted= false;
-      MDL_request mdl_request;
       bool rw_trans= (xs->rm_error != ER_XA_RBROLLBACK);
 
       if (rw_trans && thd->is_read_only_ctx())
@@ -804,10 +823,7 @@ bool trans_xa_rollback(THD *thd)
         goto _end_external_xid;
       }
 
-      MDL_REQUEST_INIT(&mdl_request, MDL_key::BACKUP, "", "", MDL_BACKUP_COMMIT,
-                       MDL_EXPLICIT);
-      if (thd->mdl_context.acquire_lock(&mdl_request,
-                                        thd->variables.lock_wait_timeout))
+      if (trans_xa_get_backup_lock(thd, &mdl_request))
       {
         /*
           We can't rollback an XA transaction on lock failure due to
@@ -818,10 +834,6 @@ bool trans_xa_rollback(THD *thd)
 
         goto _end_external_xid;
       }
-      else
-      {
-        thd->backup_commit_lock= &mdl_request;
-      }
       res= xa_trans_rolled_back(xs);
       DBUG_ASSERT(!xid_state.xid_cache_element);
 
@@ -838,11 +850,7 @@ bool trans_xa_rollback(THD *thd)
       xid_state.xid_cache_element= 0;
       if (!xid_deleted)
         xs->acquired_to_recovered();
-      if (mdl_request.ticket)
-      {
-        thd->mdl_context.release_lock(mdl_request.ticket);
-        thd->backup_commit_lock= 0;
-      }
+      trans_xa_release_backup_lock(thd);
     }
     else
       my_error(ER_XAER_NOTA, MYF(0));
@@ -859,11 +867,7 @@ bool trans_xa_rollback(THD *thd)
     DBUG_RETURN(TRUE);
   }
 
-  MDL_request mdl_request;
-  MDL_REQUEST_INIT(&mdl_request, MDL_key::BACKUP, "", "", MDL_BACKUP_COMMIT,
-      MDL_STATEMENT);
-  if (thd->mdl_context.acquire_lock(&mdl_request,
-        thd->variables.lock_wait_timeout))
+  if (trans_xa_get_backup_lock(thd, &mdl_request))
   {
     /*
       We can't rollback an XA transaction on lock failure due to
@@ -874,7 +878,9 @@ bool trans_xa_rollback(THD *thd)
     DBUG_RETURN(true);
   }
 
-  DBUG_RETURN(xa_trans_force_rollback(thd));
+  error= xa_trans_force_rollback(thd);
+  trans_xa_release_backup_lock(thd);
+  DBUG_RETURN(error);
 }
 
 
@@ -1039,17 +1045,19 @@ static my_bool xa_recover_callback(XID_cache_element *xs, Protocol *protocol,
 }
 
 
-static my_bool xa_recover_callback_short(XID_cache_element *xs,
-                                         Protocol *protocol)
+static my_bool xa_recover_callback_short(void *x, void *p)
 {
+  XID_cache_element *xs= static_cast<XID_cache_element*>(x);
+  Protocol *protocol= static_cast<Protocol*>(p);
   return xa_recover_callback(xs, protocol, xs->xid.data,
       xs->xid.gtrid_length + xs->xid.bqual_length, &my_charset_bin);
 }
 
 
-static my_bool xa_recover_callback_verbose(XID_cache_element *xs,
-                                           Protocol *protocol)
+static my_bool xa_recover_callback_verbose(void *x, void *p)
 {
+  XID_cache_element *xs= static_cast<XID_cache_element*>(x);
+  Protocol *protocol= static_cast<Protocol*>(p);
   char buf[SQL_XIDSIZE];
   uint len= get_sql_xid(&xs->xid, buf);
   return xa_recover_callback(xs, protocol, buf, len,
@@ -1089,14 +1097,14 @@ void xa_recover_get_fields(THD *thd, List<Item> *field_list,
       len= SQL_XIDSIZE;
       cs= &my_charset_utf8mb3_general_ci;
       if (action)
-	*action= (my_hash_walk_action) xa_recover_callback_verbose;
+	*action= xa_recover_callback_verbose;
     }
     else
     {
       len= XIDDATASIZE;
       cs= &my_charset_bin;
       if (action)
-	*action= (my_hash_walk_action) xa_recover_callback_short;
+	*action= xa_recover_callback_short;
     }
 
     field_list->push_back(new (mem_root)
