@@ -90,7 +90,7 @@ The commit or rollback can be seen as a subprocedure call.
 When the transaction starts to handle a rollback or commit.
 It builds a query graph which, when executed, will roll back
 or commit the incomplete transaction. The transaction
-is moved to the TRX_QUE_ROLLING_BACK or TRX_QUE_COMMITTING state.
+may be moved to the TRX_QUE_ROLLING_BACK state.
 If specified, the SQL cursors opened by the transaction are closed.
 When the execution of the graph completes, it is like returning
 from a subprocedure: the query thread which requested the operation
@@ -100,14 +100,7 @@ starts running again. */
 Creates a query graph fork node.
 @return own: fork node */
 que_fork_t*
-que_fork_create(
-/*============*/
-	que_t*		graph,		/*!< in: graph, if NULL then this
-					fork node is assumed to be the
-					graph root */
-	que_node_t*	parent,		/*!< in: parent node */
-	ulint		fork_type,	/*!< in: fork type */
-	mem_heap_t*	heap)		/*!< in: memory heap where created */
+que_fork_create(mem_heap_t *heap)
 {
 	que_fork_t*	fork;
 
@@ -117,15 +110,11 @@ que_fork_create(
 
 	fork->heap = heap;
 
-	fork->fork_type = fork_type;
-
-	fork->common.parent = parent;
-
 	fork->common.type = QUE_NODE_FORK;
 
 	fork->state = QUE_FORK_COMMAND_WAIT;
 
-	fork->graph = (graph != NULL) ? graph : fork;
+	fork->graph = fork;
 
 	UT_LIST_INIT(fork->thrs, &que_thr_t::thrs);
 
@@ -157,10 +146,6 @@ que_thr_create(
 
 	thr->common.type = QUE_NODE_THR;
 
-	thr->state = QUE_THR_COMMAND_WAIT;
-
-	thr->lock_state = QUE_THR_LOCK_NOLOCK;
-
 	thr->prebuilt = prebuilt;
 
 	UT_LIST_ADD_LAST(parent->thrs, thr);
@@ -168,45 +153,6 @@ que_thr_create(
 	return(thr);
 }
 
-/**********************************************************************//**
-Moves a suspended query thread to the QUE_THR_RUNNING state and may release
-a worker thread to execute it. This function should be used to end
-the wait state of a query thread waiting for a lock or a stored procedure
-completion.
-@return the query thread that needs to be released. */
-que_thr_t*
-que_thr_end_lock_wait(
-/*==================*/
-	trx_t*		trx)	/*!< in: transaction with que_state in
-				QUE_THR_LOCK_WAIT */
-{
-	que_thr_t*	thr;
-
-	ut_ad(lock_mutex_own());
-	ut_ad(trx_mutex_own(trx));
-
-	thr = trx->lock.wait_thr;
-
-	ut_ad(thr != NULL);
-
-	ut_ad(trx->lock.que_state == TRX_QUE_LOCK_WAIT);
-	/* In MySQL this is the only possible state here */
-	ut_a(thr->state == QUE_THR_LOCK_WAIT);
-
-	bool was_active = thr->is_active;
-
-	thr->start_running();
-
-	trx->lock.que_state = TRX_QUE_RUNNING;
-
-	trx->lock.wait_thr = NULL;
-
-	/* In MySQL we let the OS thread (not just the query thread) to wait
-	for the lock to be released: */
-
-	return((!was_active && thr != NULL) ? thr : NULL);
-}
-
 /**********************************************************************//**
 Inits a query thread for a command. */
 UNIV_INLINE
@@ -217,53 +163,7 @@ que_thr_init_command(
 {
 	thr->run_node = thr;
 	thr->prev_node = thr->common.parent;
-	thr->start_running();
-}
-
-/**********************************************************************//**
-Round robin scheduler.
-@return a query thread of the graph moved to QUE_THR_RUNNING state, or
-NULL; the query thread should be executed by que_run_threads by the
-caller */
-que_thr_t*
-que_fork_scheduler_round_robin(
-/*===========================*/
-	que_fork_t*	fork,		/*!< in: a query fork */
-	que_thr_t*	thr)		/*!< in: current pos */
-{
-	trx_mutex_enter(fork->trx);
-
-	/* If no current, start first available. */
-	if (thr == NULL) {
-		thr = UT_LIST_GET_FIRST(fork->thrs);
-	} else {
-		thr = UT_LIST_GET_NEXT(thrs, thr);
-	}
-
-	if (thr) {
-
-		fork->state = QUE_FORK_ACTIVE;
-
-		fork->last_sel_node = NULL;
-
-		switch (thr->state) {
-		case QUE_THR_COMMAND_WAIT:
-		case QUE_THR_COMPLETED:
-			ut_a(!thr->is_active);
-			que_thr_init_command(thr);
-			break;
-
-		case QUE_THR_SUSPENDED:
-		case QUE_THR_LOCK_WAIT:
-		default:
-			ut_error;
-
-		}
-	}
-
-	trx_mutex_exit(fork->trx);
-
-	return(thr);
+	thr->state = QUE_THR_RUNNING;
 }
 
 /**********************************************************************//**
@@ -279,73 +179,15 @@ que_fork_start_command(
 /*===================*/
 	que_fork_t*	fork)	/*!< in: a query fork */
 {
-	que_thr_t*	thr;
-	que_thr_t*	suspended_thr = NULL;
-	que_thr_t*	completed_thr = NULL;
-
 	fork->state = QUE_FORK_ACTIVE;
 
 	fork->last_sel_node = NULL;
 
-	suspended_thr = NULL;
-	completed_thr = NULL;
+	que_thr_t* thr = UT_LIST_GET_FIRST(fork->thrs);
 
-	/* Choose the query thread to run: usually there is just one thread,
-	but in a parallelized select, which necessarily is non-scrollable,
-	there may be several to choose from */
-
-	/* First we try to find a query thread in the QUE_THR_COMMAND_WAIT
-	state. Then we try to find a query thread in the QUE_THR_SUSPENDED
-	state, finally we try to find a query thread in the QUE_THR_COMPLETED
-	state */
-
-	/* We make a single pass over the thr list within which we note which
-	threads are ready to run. */
-	for (thr = UT_LIST_GET_FIRST(fork->thrs);
-	     thr != NULL;
-	     thr = UT_LIST_GET_NEXT(thrs, thr)) {
-
-		switch (thr->state) {
-		case QUE_THR_COMMAND_WAIT:
-
-			/* We have to send the initial message to query thread
-			to start it */
-
-			que_thr_init_command(thr);
-
-			return(thr);
-
-		case QUE_THR_SUSPENDED:
-			/* In this case the execution of the thread was
-			suspended: no initial message is needed because
-			execution can continue from where it was left */
-			if (!suspended_thr) {
-				suspended_thr = thr;
-			}
-
-			break;
-
-		case QUE_THR_COMPLETED:
-			if (!completed_thr) {
-				completed_thr = thr;
-			}
-
-			break;
-
-		case QUE_THR_RUNNING:
-		case QUE_THR_LOCK_WAIT:
-			ut_error;
-		}
-	}
-
-	if (suspended_thr) {
-		thr = suspended_thr;
-		thr->start_running();
-	} else if (completed_thr) {
-		thr = completed_thr;
+	if (thr) {
+		ut_ad(thr->state == QUE_THR_COMPLETED);
 		que_thr_init_command(thr);
-	} else {
-		ut_error;
 	}
 
 	return(thr);
@@ -360,9 +202,9 @@ que_graph_free_stat_list(
 	que_node_t*	node)	/*!< in: first query graph node in the list */
 {
 	while (node) {
+		que_node_t* next = que_node_get_next(node);
 		que_graph_free_recursive(node);
-
-		node = que_node_get_next(node);
+		node = next;
 	}
 }
 
@@ -421,19 +263,10 @@ que_graph_free_recursive(
 
 		break;
 	case QUE_NODE_INSERT:
-
 		ins = static_cast<ins_node_t*>(node);
 
 		que_graph_free_recursive(ins->select);
-		ins->select = NULL;
-
 		ins->~ins_node_t();
-
-		if (ins->entry_sys_heap != NULL) {
-			mem_heap_free(ins->entry_sys_heap);
-			ins->entry_sys_heap = NULL;
-		}
-
 		break;
 	case QUE_NODE_PURGE:
 		purge = static_cast<purge_node_t*>(node);
@@ -446,13 +279,9 @@ que_graph_free_recursive(
 	case QUE_NODE_UPDATE:
 		upd = static_cast<upd_node_t*>(node);
 
-		if (upd->in_mysql_interface) {
-
-			btr_pcur_free_for_mysql(upd->pcur);
-			upd->in_mysql_interface = false;
-		}
-
 		que_graph_free_recursive(upd->cascade_node);
+		ut_free(upd->pcur->old_rec_buf);
+		upd->pcur->old_rec_buf = NULL;
 
 		if (upd->cascade_heap) {
 			mem_heap_free(upd->cascade_heap);
@@ -550,7 +379,7 @@ que_graph_free(
 		sym_tab_free_private(graph->sym_tab);
 	}
 
-	if (graph->info && graph->info->graph_owns_us) {
+	if (graph->info) {
 		pars_info_free(graph->info);
 	}
 
@@ -580,187 +409,18 @@ que_thr_node_step(
 		return(thr);
 	}
 
-	trx_mutex_enter(thr_get_trx(thr));
+	trx_t *trx= thr->graph->trx;
+	trx->mutex_lock();
 
-	if (que_thr_peek_stop(thr)) {
-
-		trx_mutex_exit(thr_get_trx(thr));
-
-		return(thr);
-	}
-
-	/* Thread execution completed */
-
-	thr->state = QUE_THR_COMPLETED;
-
-	trx_mutex_exit(thr_get_trx(thr));
-
-	return(NULL);
-}
-
-/**********************************************************************//**
-Stops a query thread if graph or trx is in a state requiring it. The
-conditions are tested in the order (1) graph, (2) trx.
-@return TRUE if stopped */
-ibool
-que_thr_stop(
-/*=========*/
-	que_thr_t*	thr)	/*!< in: query thread */
-{
-	que_t*		graph;
-	trx_t*		trx = thr_get_trx(thr);
-
-	graph = thr->graph;
-
-	ut_ad(trx_mutex_own(trx));
-
-	if (graph->state == QUE_FORK_COMMAND_WAIT) {
-
-		thr->state = QUE_THR_SUSPENDED;
-
-	} else if (trx->lock.que_state == TRX_QUE_LOCK_WAIT) {
-
-		trx->lock.wait_thr = thr;
-		thr->state = QUE_THR_LOCK_WAIT;
-
-	} else if (trx->error_state != DB_SUCCESS
-		   && trx->error_state != DB_LOCK_WAIT) {
-
-		/* Error handling built for the MySQL interface */
+	if (!trx->lock.wait_thr && thr->graph->state == QUE_FORK_ACTIVE) {
 		thr->state = QUE_THR_COMPLETED;
-
-	} else if (graph->fork_type == QUE_FORK_ROLLBACK) {
-
-		thr->state = QUE_THR_SUSPENDED;
-	} else {
-		ut_ad(graph->state == QUE_FORK_ACTIVE);
-
-		return(FALSE);
+		thr = NULL;
 	}
 
-	return(TRUE);
+	trx->mutex_unlock();
+	return(thr);
 }
 
-/**********************************************************************//**
-Decrements the query thread reference counts in the query graph and the
-transaction.
-*** NOTE ***:
-This and que_thr_stop_for_mysql are the only functions where the reference
-count can be decremented and this function may only be called from inside
-que_run_threads! These restrictions exist to make the rollback code easier
-to maintain. */
-static
-void
-que_thr_dec_refer_count(
-/*====================*/
-	que_thr_t*	thr,		/*!< in: query thread */
-	que_thr_t**	next_thr)	/*!< in/out: next query thread to run;
-					if the value which is passed in is
-					a pointer to a NULL pointer, then the
-					calling function can start running
-					a new query thread */
-{
-	trx_t*		trx;
-
-	trx = thr_get_trx(thr);
-
-	ut_a(thr->is_active);
-	ut_ad(trx_mutex_own(trx));
-
-	if (thr->state == QUE_THR_RUNNING) {
-
-		if (!que_thr_stop(thr)) {
-
-			ut_a(next_thr != NULL && *next_thr == NULL);
-
-			/* The reason for the thr suspension or wait was
-			already canceled before we came here: continue
-			running the thread.
-
-			This is also possible because in trx_commit_step() we
-			assume a single query thread. We set the query thread
-			state to QUE_THR_RUNNING. */
-
-			/* fprintf(stderr,
-		       		"Wait already ended: trx: %p\n", trx); */
-
-			/* Normally srv_suspend_mysql_thread resets
-			the state to DB_SUCCESS before waiting, but
-			in this case we have to do it here,
-			otherwise nobody does it. */
-
-			trx->error_state = DB_SUCCESS;
-
-			*next_thr = thr;
-
-			return;
-		}
-	}
-
-	ut_d(static_cast<que_fork_t*>(thr->common.parent)->set_active(false));
-	thr->is_active = false;
-}
-
-/**********************************************************************//**
-A patch for MySQL used to 'stop' a dummy query thread used in MySQL. The
-query thread is stopped and made inactive, except in the case where
-it was put to the lock wait state in lock0lock.cc, but the lock has already
-been granted or the transaction chosen as a victim in deadlock resolution. */
-void
-que_thr_stop_for_mysql(
-/*===================*/
-	que_thr_t*	thr)	/*!< in: query thread */
-{
-	trx_t*	trx;
-
-	trx = thr_get_trx(thr);
-
-	trx_mutex_enter(trx);
-
-	if (thr->state == QUE_THR_RUNNING) {
-
-		if (trx->error_state != DB_SUCCESS
-		    && trx->error_state != DB_LOCK_WAIT) {
-
-			/* Error handling built for the MySQL interface */
-			thr->state = QUE_THR_COMPLETED;
-		} else {
-			/* It must have been a lock wait but the lock was
-			already released, or this transaction was chosen
-			as a victim in selective deadlock resolution */
-
-			trx_mutex_exit(trx);
-
-			return;
-		}
-	}
-
-	ut_ad(thr->is_active);
-	ut_d(thr->set_active(false));
-	thr->is_active= false;
-
-	trx_mutex_exit(trx);
-}
-
-#ifdef UNIV_DEBUG
-/** Change the 'active' status */
-void que_fork_t::set_active(bool active)
-{
-  if (active)
-  {
-    n_active_thrs++;
-    trx->lock.n_active_thrs++;
-  }
-  else
-  {
-    ut_ad(n_active_thrs);
-    ut_ad(trx->lock.n_active_thrs);
-    n_active_thrs--;
-    trx->lock.n_active_thrs--;
-  }
-}
-#endif
-
 /****************************************************************//**
 Get the first containing loop node (e.g. while_node_t or for_node_t) for the
 given node, or NULL if the node is not within a loop.
@@ -791,6 +451,26 @@ que_node_get_containing_loop_node(
 	return(node);
 }
 
+/**********************************************************************//**
+Performs an execution step of an open or close cursor statement node.
+@param thr query thread */
+static void open_step(que_thr_t *thr)
+{
+  open_node_t *node= static_cast<open_node_t*>(thr->run_node);
+  ut_ad(que_node_get_type(node) == QUE_NODE_OPEN);
+  sel_node_t *sel_node= node->cursor_def;
+
+  if (node->op_type == ROW_SEL_OPEN_CURSOR)
+    sel_node->state= SEL_NODE_OPEN;
+  else
+  {
+    ut_ad(sel_node->state != SEL_NODE_CLOSED);
+    sel_node->state= SEL_NODE_CLOSED;
+  }
+
+  thr->run_node= que_node_get_parent(node);
+}
+
 /**********************************************************************//**
 Performs an execution step on a query thread.
 @return query thread to run next: it may differ from the input
@@ -834,8 +514,7 @@ que_thr_step(
 			for_step(thr);
 		} else if (type == QUE_NODE_PROC) {
 			if (thr->prev_node == que_node_get_parent(node)) {
-				trx->last_sql_stat_start.least_undo_no
-					= trx->undo_no;
+				trx->last_stmt_start = trx->undo_no;
 			}
 
 			proc_step(thr);
@@ -857,7 +536,7 @@ que_thr_step(
 	} else if (type == QUE_NODE_FETCH) {
 		thr = fetch_step(thr);
 	} else if (type == QUE_NODE_OPEN) {
-		thr = open_step(thr);
+		open_step(thr);
 	} else if (type == QUE_NODE_FUNC) {
 		proc_eval_step(thr);
 
@@ -909,19 +588,14 @@ que_run_threads_low(
 /*================*/
 	que_thr_t*	thr)	/*!< in: query thread */
 {
-	trx_t*		trx;
-	que_thr_t*	next_thr;
-
 	ut_ad(thr->state == QUE_THR_RUNNING);
-	ut_a(thr_get_trx(thr)->error_state == DB_SUCCESS);
-	ut_ad(!trx_mutex_own(thr_get_trx(thr)));
 
 	/* cumul_resource counts how much resources the OS thread (NOT the
 	query thread) has spent in this function */
 
-	trx = thr_get_trx(thr);
-
-	do {
+	for (trx_t* trx = thr_get_trx(thr);;) {
+		ut_ad(!trx->mutex_is_owner());
+		ut_a(trx->error_state == DB_SUCCESS);
 		/* Check that there is enough space in the log to accommodate
 		possible log entries by this query step; if the operation can
 		touch more than about 4 pages, checks must be made also within
@@ -932,33 +606,14 @@ que_run_threads_low(
 		/* Perform the actual query step: note that the query thread
 		may change if, e.g., a subprocedure call is made */
 
-		/*-------------------------*/
-		next_thr = que_thr_step(thr);
-		/*-------------------------*/
-
-		trx_mutex_enter(trx);
-
-		ut_a(next_thr == NULL || trx->error_state == DB_SUCCESS);
-
-		if (next_thr != thr) {
-			ut_a(next_thr == NULL);
-
-			/* This can change next_thr to a non-NULL value
-			if there was a lock wait that already completed. */
-
-			que_thr_dec_refer_count(thr, &next_thr);
-
-			if (next_thr != NULL) {
-
-				thr = next_thr;
-			}
+		que_thr_t* next_thr = que_thr_step(thr);
+		ut_ad(trx == thr_get_trx(thr));
+		if (!next_thr) {
+			return;
 		}
 
-		ut_ad(trx == thr_get_trx(thr));
-
-		trx_mutex_exit(trx);
-
-	} while (next_thr != NULL);
+		ut_a(next_thr == thr);
+	}
 }
 
 /**********************************************************************//**
@@ -968,47 +623,20 @@ que_run_threads(
 /*============*/
 	que_thr_t*	thr)	/*!< in: query thread */
 {
-	ut_ad(!trx_mutex_own(thr_get_trx(thr)));
-
+	trx_t* trx = thr->graph->trx;
 loop:
-	ut_a(thr_get_trx(thr)->error_state == DB_SUCCESS);
-
+	ut_a(trx->error_state == DB_SUCCESS);
 	que_run_threads_low(thr);
 
-	switch (thr->state) {
-
-	case QUE_THR_RUNNING:
-		/* There probably was a lock wait, but it already ended
-		before we came here: continue running thr */
-
-		goto loop;
-
-	case QUE_THR_LOCK_WAIT:
-		lock_wait_suspend_thread(thr);
-
-		trx_mutex_enter(thr_get_trx(thr));
-
-		ut_a(thr_get_trx(thr)->id != 0);
-
-		if (thr_get_trx(thr)->error_state != DB_SUCCESS) {
-			/* thr was chosen as a deadlock victim or there was
-			a lock wait timeout */
-
-			que_thr_dec_refer_count(thr, NULL);
-			trx_mutex_exit(thr_get_trx(thr));
-			break;
+	if (thr->state != QUE_THR_COMPLETED) {
+		if (trx->lock.wait_thr) {
+			ut_ad(trx->id);
+			if (lock_wait(thr) == DB_SUCCESS) {
+				goto loop;
+			}
+		} else if (trx->error_state == DB_SUCCESS) {
+			goto loop;
 		}
-
-		trx_mutex_exit(thr_get_trx(thr));
-		goto loop;
-
-	case QUE_THR_COMPLETED:
-	case QUE_THR_COMMAND_WAIT:
-		/* Do nothing */
-		break;
-
-	default:
-		ut_error;
 	}
 }
 
@@ -1020,9 +648,6 @@ que_eval_sql(
 /*=========*/
 	pars_info_t*	info,	/*!< in: info struct, or NULL */
 	const char*	sql,	/*!< in: SQL string */
-	bool		reserve_dict_mutex,
-				/*!< in: whether to acquire/release
-				dict_sys.mutex around call to pars_sql. */
 	trx_t*		trx)	/*!< in: trx */
 {
 	que_thr_t*	thr;
@@ -1033,34 +658,16 @@ que_eval_sql(
 
 	ut_a(trx->error_state == DB_SUCCESS);
 
-	if (reserve_dict_mutex) {
-		mutex_enter(&dict_sys.mutex);
-	}
-
 	graph = pars_sql(info, sql);
 
-	if (reserve_dict_mutex) {
-		mutex_exit(&dict_sys.mutex);
-	}
-
 	graph->trx = trx;
 	trx->graph = NULL;
 
-	graph->fork_type = QUE_FORK_MYSQL_INTERFACE;
-
 	ut_a(thr = que_fork_start_command(graph));
 
 	que_run_threads(thr);
 
-	if (reserve_dict_mutex) {
-		mutex_enter(&dict_sys.mutex);
-	}
-
 	que_graph_free(graph);
 
-	if (reserve_dict_mutex) {
-		mutex_exit(&dict_sys.mutex);
-	}
-
 	DBUG_RETURN(trx->error_state);
 }
