@@ -91,10 +91,6 @@ Created 2/16/1996 Heikki Tuuri
 #include "zlib.h"
 #include "log.h"
 
-/** We are prepared for a situation that we have this many threads waiting for
-a transactional lock inside InnoDB. srv_start() sets the value. */
-ulint srv_max_n_threads;
-
 /** Log sequence number at shutdown */
 lsn_t	srv_shutdown_lsn;
 
@@ -179,7 +175,7 @@ static dberr_t create_log_file(bool create_new_db, lsn_t lsn)
 
 	/* We will retain ib_logfile0 until we have written a new logically
 	empty log as ib_logfile101 and atomically renamed it to
-	ib_logfile0 in log_t::rename_resized(). */
+	ib_logfile0 in log_t::resize_rename(). */
 	delete_log_files();
 
 	ut_ad(!os_aio_pending_reads());
@@ -201,8 +197,8 @@ static dberr_t create_log_file(bool create_new_db, lsn_t lsn)
 	bool ret;
 	os_file_t file{
           os_file_create_func(logfile0.c_str(),
-                              OS_FILE_CREATE | OS_FILE_ON_ERROR_NO_EXIT,
-                              OS_FILE_NORMAL, OS_LOG_FILE, false, &ret)
+                              OS_FILE_CREATE,
+                              OS_LOG_FILE, false, &ret)
         };
 
 	if (!ret) {
@@ -226,7 +222,12 @@ close_and_exit:
 	if (!log_sys.attach(file, srv_log_file_size)) {
 		goto close_and_exit;
 	}
-	if (!fil_system.sys_space->open(create_new_db)) {
+
+	mysql_mutex_lock(&recv_sys.mutex);
+	const bool all_opened = fil_system.sys_space->open(create_new_db);
+	mysql_mutex_unlock(&recv_sys.mutex);
+
+	if (!all_opened) {
 		goto err_exit;
 	}
 
@@ -242,7 +243,16 @@ close_and_exit:
 		srv_startup_is_before_trx_rollback_phase = false;
 	}
 
-	/* Enable checkpoints in buf_flush_page_cleaner(). */
+	/* Enable log_checkpoint() in buf_flush_page_cleaner().
+	If we are upgrading or resizing the log at startup, we must not
+	write any FILE_MODIFY or FILE_CHECKPOINT records to the old log file
+	because the ib_logfile0 could be in an old format that we can only
+	check for emptiness, or the write could lead to an overflow.
+
+	At this point, we are not holding recv_sys.mutex, but there are no
+	pending page reads, and buf_pool.flush_list is empty. Therefore,
+	we can clear the flag without risking any race condition with
+	buf_page_t::read_complete(). */
 	recv_sys.recovery_on = false;
 	log_sys.latch.wr_unlock();
 
@@ -286,7 +296,7 @@ static dberr_t srv_undo_tablespace_create(const char* name)
 		innodb_data_file_key,
 		name,
 		srv_read_only_mode ? OS_FILE_OPEN : OS_FILE_CREATE,
-		OS_FILE_NORMAL, OS_DATA_FILE, srv_read_only_mode, &ret);
+		OS_DATA_FILE, srv_read_only_mode, &ret);
 
 	if (!ret) {
 		if (os_file_get_last_error(false) != OS_FILE_ALREADY_EXISTS
@@ -374,6 +384,11 @@ inline dberr_t trx_sys_t::reset_page(mtr_t *mtr)
       sys_header->page.frame + TRX_SYS_DOUBLEWRITE
       + FSEG_HEADER_SIZE + TRX_SYS_DOUBLEWRITE_REPEAT,
       sys_header->page.frame + TRX_SYS_DOUBLEWRITE + FSEG_HEADER_SIZE, 12);
+    mtr->write<4>(
+      *sys_header,
+      TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_SPACE_ID_STORED +
+      sys_header->page.frame,
+      TRX_SYS_DOUBLEWRITE_SPACE_ID_STORED_N);
   }
 
   return DB_SUCCESS;
@@ -456,8 +471,7 @@ ATTRIBUTE_COLD static dberr_t srv_undo_tablespaces_reinit()
       buf_block_t *block= buf_page_get_gen(
         rseg->page_id(), 0, RW_X_LATCH, nullptr, BUF_GET, &mtr);
       if (!block) break;
-      while (!fseg_free_step(TRX_RSEG + TRX_RSEG_FSEG_HEADER +
-                             block->page.frame, &mtr));
+      while (!fseg_free_step(block, TRX_RSEG + TRX_RSEG_FSEG_HEADER, &mtr));
     }
   }
 
@@ -632,37 +646,49 @@ static uint32_t srv_undo_tablespace_open(bool create, const char* name,
     }
   }
 
-  pfs_os_file_t fh= os_file_create(innodb_data_file_key, name, OS_FILE_OPEN |
-                                   OS_FILE_ON_ERROR_NO_EXIT |
-                                   OS_FILE_ON_ERROR_SILENT,
-                                   OS_FILE_AIO, OS_DATA_FILE,
+  pfs_os_file_t fh= os_file_create(innodb_data_file_key, name,
+                                   OS_FILE_OPEN_SILENT,
+                                   OS_DATA_FILE,
                                    srv_read_only_mode, &success);
 
   if (!success)
     return 0;
 
+  ulint n_retries = 5;
   os_offset_t size= os_file_get_size(fh);
   ut_a(size != os_offset_t(-1));
+  page_t *apage= nullptr;
+  const page_t *page= nullptr;
 
   if (!create)
   {
-    page_t *page= static_cast<byte*>(aligned_malloc(srv_page_size,
-                                                    srv_page_size));
-    if (os_file_read(IORequestRead, fh, page, 0, srv_page_size, nullptr) !=
+    apage= static_cast<byte*>(aligned_malloc(srv_page_size, srv_page_size));
+undo_retry:
+    if (os_file_read(IORequestRead, fh, apage, 0, srv_page_size, nullptr) !=
         DB_SUCCESS)
     {
 err_exit:
-      ib::error() << "Unable to read first page of file " << name;
-      aligned_free(page);
+      if (n_retries && srv_operation == SRV_OPERATION_BACKUP)
+      {
+        sql_print_information("InnoDB: Retrying to read undo "
+                              "tablespace %s", name);
+        n_retries--;
+        goto undo_retry;
+      }
+      sql_print_error("InnoDB: Unable to read first page of file %s", name);
+      aligned_free(apage);
       return ~0U;
     }
 
+    page= apage;
+    DBUG_EXECUTE_IF("undo_space_read_fail", goto err_exit;);
+
     uint32_t id= mach_read_from_4(FIL_PAGE_SPACE_ID + page);
     if (id == 0 || id >= SRV_SPACE_ID_UPPER_BOUND ||
         memcmp_aligned<2>(FIL_PAGE_SPACE_ID + page,
                           FSP_HEADER_OFFSET + FSP_SPACE_ID + page, 4))
     {
-      ib::error() << "Inconsistent tablespace ID in file " << name;
+      sql_print_error("InnoDB: Inconsistent tablespace ID in file %s", name);
       goto err_exit;
     }
 
@@ -671,13 +697,21 @@ err_exit:
 
     if (buf_page_is_corrupted(false, page, fsp_flags))
     {
-      sql_print_error("InnoDB: Checksum mismatch in the first page of file %s",
-                      name);
-      if (recv_sys.dblwr.restore_first_page(space_id, name, fh))
+      page= recv_sys.dblwr.find_page(page_id_t{space_id, 0}, LSN_MAX);
+      if (!page)
+      {
+        /* If the first page of the given user tablespace is not there
+        in the doublewrite buffer, then the recovery is going to fail
+        now. Report error only when doublewrite buffer is not empty */
+        sql_print_error("InnoDB: Corrupted page "
+                        "[page id: space=" UINT32PF ", page number=0]"
+                        " of datafile '%s' could not be found"
+                        " in the doublewrite buffer", space_id, name);
         goto err_exit;
-    }
+      }
 
-    aligned_free(page);
+      fsp_flags= mach_read_from_4(FSP_HEADER_OFFSET + FSP_SPACE_FLAGS + page);
+    }
   }
 
   /* Load the tablespace into InnoDB's internal data structures. */
@@ -689,8 +723,7 @@ err_exit:
   fil_set_max_space_id_if_bigger(space_id);
 
   mysql_mutex_lock(&fil_system.mutex);
-  fil_space_t *space= fil_space_t::create(space_id, fsp_flags,
-                                          FIL_TYPE_TABLESPACE, nullptr,
+  fil_space_t *space= fil_space_t::create(space_id, fsp_flags, false, nullptr,
                                           FIL_ENCRYPTION_DEFAULT, true);
   ut_ad(space);
   fil_node_t *file= space->add(name, fh, 0, false, true);
@@ -700,7 +733,7 @@ err_exit:
     space->set_sizes(SRV_UNDO_TABLESPACE_SIZE_IN_PAGES);
     space->size= file->size= uint32_t(size >> srv_page_size_shift);
   }
-  else if (!file->read_page0())
+  else if (!file->read_page0(page, true))
   {
     os_file_close(file->handle);
     file->handle= OS_FILE_CLOSED;
@@ -709,6 +742,7 @@ err_exit:
   }
 
   mysql_mutex_unlock(&fil_system.mutex);
+  aligned_free(apage);
   return space_id;
 }
 
@@ -731,10 +765,7 @@ srv_check_undo_redo_logs_exists()
 
 		fh = os_file_create_func(
 			name,
-			OS_FILE_OPEN_RETRY
-			| OS_FILE_ON_ERROR_NO_EXIT
-			| OS_FILE_ON_ERROR_SILENT,
-			OS_FILE_NORMAL,
+			OS_FILE_OPEN_RETRY_SILENT,
 			OS_DATA_FILE,
 			srv_read_only_mode,
 			&ret);
@@ -755,9 +786,8 @@ srv_check_undo_redo_logs_exists()
 	auto logfilename = get_log_file_path();
 
 	fh = os_file_create_func(logfilename.c_str(),
-				 OS_FILE_OPEN_RETRY | OS_FILE_ON_ERROR_NO_EXIT
-				 | OS_FILE_ON_ERROR_SILENT,
-				 OS_FILE_NORMAL, OS_LOG_FILE,
+				 OS_FILE_OPEN_RETRY_SILENT,
+				 OS_LOG_FILE,
 				 srv_read_only_mode, &ret);
 
 	if (ret) {
@@ -819,7 +849,7 @@ unused_undo:
   {
      char name[OS_FILE_MAX_PATH];
      snprintf(name, sizeof name, "%s/undo%03u", srv_undo_dir, i);
-     uint32_t space_id= srv_undo_tablespace_open(create_new_undo, name, i);
+     uint32_t space_id= srv_undo_tablespace_open(false, name, i);
      if (!space_id || space_id == ~0U)
        break;
      if (0 == srv_undo_tablespaces_open++)
@@ -870,11 +900,12 @@ dberr_t srv_undo_tablespaces_init(bool create_new_undo, mtr_t *mtr)
   srv_undo_tablespaces_active= srv_undo_tablespaces;
 
   uint32_t n_undo= (create_new_undo || srv_operation == SRV_OPERATION_BACKUP ||
-                 srv_operation == SRV_OPERATION_RESTORE_DELTA)
+                    srv_operation == SRV_OPERATION_RESTORE_DELTA)
     ? srv_undo_tablespaces : TRX_SYS_N_RSEGS;
 
-  if (dberr_t err= srv_all_undo_tablespaces_open(create_new_undo, n_undo))
-    return err;
+  mysql_mutex_lock(&recv_sys.mutex);
+  dberr_t err= srv_all_undo_tablespaces_open(create_new_undo, n_undo);
+  mysql_mutex_unlock(&recv_sys.mutex);
 
   /* Initialize srv_undo_space_id_start=0 when there are no
   dedicated undo tablespaces. */
@@ -882,16 +913,11 @@ dberr_t srv_undo_tablespaces_init(bool create_new_undo, mtr_t *mtr)
     srv_undo_space_id_start= 0;
 
   if (create_new_undo)
-  {
-    for (uint32_t i= 0; i < srv_undo_tablespaces; ++i)
-    {
-      dberr_t err= fsp_header_init(fil_space_get(srv_undo_space_id_start + i),
-                                   SRV_UNDO_TABLESPACE_SIZE_IN_PAGES, mtr);
-      if (err) return err;
-    }
-  }
+    for (uint32_t i= 0; err == DB_SUCCESS && i < srv_undo_tablespaces; ++i)
+      err= fsp_header_init(fil_space_get(srv_undo_space_id_start + i),
+                           SRV_UNDO_TABLESPACE_SIZE_IN_PAGES, mtr);
 
-  return DB_SUCCESS;
+  return err;
 }
 
 /** Create the temporary file tablespace.
@@ -1031,7 +1057,7 @@ srv_init_abort_low(
 /** Prepare to delete the redo log file. Flush the dirty pages from all the
 buffer pools.  Flush the redo log buffer to the redo log file.
 @return lsn upto which data pages have been flushed. */
-static lsn_t srv_prepare_to_delete_redo_log_file()
+static lsn_t srv_prepare_to_delete_redo_log_file() noexcept
 {
   DBUG_ENTER("srv_prepare_to_delete_redo_log_file");
 
@@ -1045,7 +1071,7 @@ static lsn_t srv_prepare_to_delete_redo_log_file()
 
   log_sys.latch.wr_lock(SRW_LOCK_CALL);
   const bool latest_format{log_sys.is_latest()};
-  lsn_t flushed_lsn{log_sys.get_lsn()};
+  lsn_t flushed_lsn{log_sys.get_flushed_lsn(std::memory_order_relaxed)};
 
   if (latest_format && !(log_sys.file_size & 4095) &&
       flushed_lsn != log_sys.next_checkpoint_lsn +
@@ -1053,6 +1079,11 @@ static lsn_t srv_prepare_to_delete_redo_log_file()
        ? SIZE_OF_FILE_CHECKPOINT + 8
        : SIZE_OF_FILE_CHECKPOINT))
   {
+#ifdef HAVE_PMEM
+    if (!log_sys.is_opened())
+      log_sys.buf_size= unsigned(std::min<uint64_t>(log_sys.capacity(),
+                                                    log_sys.buf_size_max));
+#endif
     fil_names_clear(flushed_lsn);
     flushed_lsn= log_sys.get_lsn();
   }
@@ -1090,9 +1121,10 @@ same_size:
 
   log_sys.latch.wr_unlock();
 
-  log_write_up_to(flushed_lsn, false);
+  if (latest_format)
+    log_write_up_to(flushed_lsn, false);
 
-  ut_ad(flushed_lsn == log_sys.get_lsn());
+  ut_ad(flushed_lsn == log_get_lsn());
   ut_ad(!os_aio_pending_reads());
   ut_d(mysql_mutex_lock(&buf_pool.flush_list_mutex));
   ut_ad(!buf_pool.get_oldest_modification(0));
@@ -1107,6 +1139,18 @@ static tpool::task rollback_all_recovered_task(trx_rollback_all_recovered,
 					       nullptr,
 					       &rollback_all_recovered_group);
 
+inline lsn_t log_t::init_lsn() noexcept
+{
+  latch.wr_lock(SRW_LOCK_CALL);
+  ut_ad(!write_lsn_offset);
+  write_lsn_offset= 0;
+  const lsn_t lsn{base_lsn.load(std::memory_order_relaxed)};
+  flushed_to_disk_lsn.store(lsn, std::memory_order_relaxed);
+  write_lsn= lsn;
+  latch.wr_unlock();
+  return lsn;
+}
+
 /** Start InnoDB.
 @param[in]	create_new_db	whether to create a new database
 @return DB_SUCCESS or error code */
@@ -1179,12 +1223,6 @@ dberr_t srv_start(bool create_new_db)
 	mysql_stage_register("innodb", srv_stages,
 			     static_cast<int>(UT_ARR_SIZE(srv_stages)));
 
-	srv_max_n_threads =
-		1 /* dict_stats_thread */
-		+ 1 /* fts_optimize_thread */
-		+ 128 /* safety margin */
-		+ max_connections;
-
 	srv_boot();
 
 	ib::info() << my_crc32c_implementation();
@@ -1267,37 +1305,13 @@ dberr_t srv_start(bool create_new_db)
 
 	fil_system.create(srv_file_per_table ? 50000 : 5000);
 
-	ib::info() << "Initializing buffer pool, total size = "
-		<< ib::bytes_iec{srv_buf_pool_size}
-		<< ", chunk size = " << ib::bytes_iec{srv_buf_pool_chunk_unit};
-
 	if (buf_pool.create()) {
-		ib::error() << "Cannot allocate memory for the buffer pool";
-
 		return(srv_init_abort(DB_ERROR));
 	}
 
-	ib::info() << "Completed initialization of buffer pool";
-
-#ifdef UNIV_DEBUG
-	/* We have observed deadlocks with a 5MB buffer pool but
-	the actual lower limit could very well be a little higher. */
-
-	if (srv_buf_pool_size <= 5 * 1024 * 1024) {
-
-		ib::info() << "Small buffer pool size ("
-			<< ib::bytes_iec{srv_buf_pool_size}
-			<< "), the flst_validate() debug function can cause a"
-			<< " deadlock if the buffer pool fills up.";
-	}
-#endif /* UNIV_DEBUG */
-
-	if (!log_sys.create()) {
-		return srv_init_abort(DB_ERROR);
-	}
-
+	log_sys.create();
 	recv_sys.create();
-	lock_sys.create(srv_lock_table_size);
+	lock_sys.create(srv_lock_table_size = 5 * buf_pool.curr_size());
 
 	srv_startup_is_before_trx_rollback_phase = true;
 
@@ -1370,9 +1384,14 @@ dberr_t srv_start(bool create_new_db)
 
 	/* Open data files in the system tablespace: we keep
 	them open until database shutdown */
+	mysql_mutex_lock(&recv_sys.mutex);
 	ut_d(fil_system.sys_space->recv_size = srv_sys_space_size_debug);
+	err = fil_system.sys_space->open(create_new_db)
+		? DB_SUCCESS : DB_ERROR;
+	mysql_mutex_unlock(&recv_sys.mutex);
 
-	if (fil_system.sys_space->open(create_new_db)) {
+
+	if (err == DB_SUCCESS) {
 		mtr_t mtr;
 		mtr.start();
 		err= srv_undo_tablespaces_init(create_new_db, &mtr);
@@ -1458,8 +1477,6 @@ dberr_t srv_start(bool create_new_db)
 		err = recv_recovery_from_checkpoint_start();
 		recv_sys.close_files();
 
-		recv_sys.dblwr.pages.clear();
-
 		if (err != DB_SUCCESS) {
 			return(srv_init_abort(err));
 		}
@@ -1523,6 +1540,71 @@ dberr_t srv_start(bool create_new_db)
 
 		fil_system.space_id_reuse_warned = false;
 
+		if (srv_operation > SRV_OPERATION_EXPORT_RESTORED) {
+			ut_ad(srv_operation == SRV_OPERATION_RESTORE_EXPORT
+			      || srv_operation == SRV_OPERATION_RESTORE);
+			return(err);
+		}
+
+		/* Upgrade or resize or rebuild the redo logs before
+		generating any dirty pages, so that the old redo log
+		file will not be written to. */
+
+		if (srv_force_recovery == SRV_FORCE_NO_LOG_REDO) {
+			/* Completely ignore the redo log. */
+		} else if (srv_read_only_mode) {
+			/* Leave the redo log alone. */
+		} else if (log_sys.file_size == srv_log_file_size
+			   && log_sys.format
+			   == (srv_encrypt_log
+			       ? log_t::FORMAT_ENC_10_8
+			       : log_t::FORMAT_10_8)) {
+			/* No need to add or remove encryption,
+			upgrade, or resize. */
+			delete_log_files();
+		} else {
+			/* Prepare to delete the old redo log file */
+			const lsn_t lsn{srv_prepare_to_delete_redo_log_file()};
+
+			DBUG_EXECUTE_IF("innodb_log_abort_1",
+					return(srv_init_abort(DB_ERROR)););
+			/* Prohibit redo log writes from any other
+			threads until creating a log checkpoint at the
+			end of create_log_file(). */
+			ut_d(recv_no_log_write = true);
+			ut_ad(!os_aio_pending_reads());
+			ut_d(mysql_mutex_lock(&buf_pool.flush_list_mutex));
+			ut_ad(!buf_pool.get_oldest_modification(0));
+			ut_d(mysql_mutex_unlock(&buf_pool.flush_list_mutex));
+			/* os_aio_pending_writes() may hold here if
+			some write_io_callback() did not release the
+			slot yet. However, the page write itself must
+			have completed, because the buf_pool.flush_list
+			is empty. In debug builds, we wait for this to
+			happen, hoping to get a hung process if this
+			assumption does not hold. */
+			ut_d(os_aio_wait_until_no_pending_writes(false));
+
+			/* Close the redo log file, so that we can replace it */
+			log_sys.close_file();
+
+			DBUG_EXECUTE_IF("innodb_log_abort_5",
+					return(srv_init_abort(DB_ERROR)););
+			DBUG_PRINT("ib_log", ("After innodb_log_abort_5"));
+
+			err = create_log_file(false, lsn);
+
+			if (err == DB_SUCCESS && log_sys.resize_rename()) {
+				err = DB_ERROR;
+			}
+
+			if (err != DB_SUCCESS) {
+				return(srv_init_abort(err));
+			}
+		}
+
+		recv_sys.debug_free();
+
 		if (!srv_read_only_mode) {
 			const uint32_t flags = FSP_FLAGS_PAGE_SSIZE();
 			for (uint32_t id = srv_undo_space_id_start;
@@ -1607,71 +1689,6 @@ dberr_t srv_start(bool create_new_db)
 				return(srv_init_abort(DB_ERROR));
 			}
 		}
-
-		if (srv_operation > SRV_OPERATION_EXPORT_RESTORED) {
-			ut_ad(srv_operation == SRV_OPERATION_RESTORE_EXPORT
-			      || srv_operation == SRV_OPERATION_RESTORE);
-			return(err);
-		}
-
-		/* Upgrade or resize or rebuild the redo logs before
-		generating any dirty pages, so that the old redo log
-		file will not be written to. */
-
-		if (srv_force_recovery == SRV_FORCE_NO_LOG_REDO) {
-			/* Completely ignore the redo log. */
-		} else if (srv_read_only_mode) {
-			/* Leave the redo log alone. */
-		} else if (log_sys.file_size == srv_log_file_size
-			   && log_sys.format
-			   == (srv_encrypt_log
-			       ? log_t::FORMAT_ENC_10_8
-			       : log_t::FORMAT_10_8)) {
-			/* No need to add or remove encryption,
-			upgrade, or resize. */
-			delete_log_files();
-		} else {
-			/* Prepare to delete the old redo log file */
-			const lsn_t lsn{srv_prepare_to_delete_redo_log_file()};
-
-			DBUG_EXECUTE_IF("innodb_log_abort_1",
-					return(srv_init_abort(DB_ERROR)););
-			/* Prohibit redo log writes from any other
-			threads until creating a log checkpoint at the
-			end of create_log_file(). */
-			ut_d(recv_no_log_write = true);
-			ut_ad(!os_aio_pending_reads());
-			ut_d(mysql_mutex_lock(&buf_pool.flush_list_mutex));
-			ut_ad(!buf_pool.get_oldest_modification(0));
-			ut_d(mysql_mutex_unlock(&buf_pool.flush_list_mutex));
-			/* os_aio_pending_writes() may hold here if
-			some write_io_callback() did not release the
-			slot yet. However, the page write itself must
-			have completed, because the buf_pool.flush_list
-			is empty. In debug builds, we wait for this to
-			happen, hoping to get a hung process if this
-			assumption does not hold. */
-			ut_d(os_aio_wait_until_no_pending_writes(false));
-
-			/* Close the redo log file, so that we can replace it */
-			log_sys.close_file();
-
-			DBUG_EXECUTE_IF("innodb_log_abort_5",
-					return(srv_init_abort(DB_ERROR)););
-			DBUG_PRINT("ib_log", ("After innodb_log_abort_5"));
-
-			err = create_log_file(false, lsn);
-
-			if (err == DB_SUCCESS && log_sys.resize_rename()) {
-				err = DB_ERROR;
-			}
-
-			if (err != DB_SUCCESS) {
-				return(srv_init_abort(err));
-			}
-		}
-
-		recv_sys.debug_free();
 	}
 
 	ut_ad(err == DB_SUCCESS);
@@ -1858,15 +1875,11 @@ skip_monitors:
 	if (srv_print_verbose_log) {
 		sql_print_information("InnoDB: "
 				      "log sequence number " LSN_PF
-#ifdef HAVE_PMEM
 				      "%s"
-#endif
 				      "; transaction id " TRX_ID_FMT,
 				      recv_sys.lsn,
-#ifdef HAVE_PMEM
-				      log_sys.is_pmem()
+				      log_sys.is_mmap()
 				      ? " (memory-mapped)" : "",
-#endif
 				      trx_sys.get_max_trx_id());
 	}
 
