@@ -9,11 +9,11 @@ use My::Find;
 use File::Basename;
 use strict;
 
-sub is_default { not $::opt_embedded_server }
+#sub is_default { not $::opt_embedded_server }
 
 my $sst_dump=
 ::mtr_exe_maybe_exists(
-  "$::bindir/storage/rocksdb$::opt_vs_config/sst_dump",
+  "$::bindir/storage/rocksdb$::multiconfig/sst_dump",
   "$::path_client_bindir/sst_dump");
 return "RocksDB is not compiled, no sst_dump" unless $sst_dump;
 $ENV{MARIAROCKS_SST_DUMP}="$sst_dump";
