@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301 USA */
 #pragma once
 
 namespace myrocks {
@@ -34,4 +34,4 @@ extern struct st_maria_plugin rdb_i_s_index_file_map;
 extern struct st_maria_plugin rdb_i_s_lock_info;
 extern struct st_maria_plugin rdb_i_s_trx_info;
 extern struct st_maria_plugin rdb_i_s_deadlock_info;
-} // namespace myrocks
+}  // namespace myrocks
