@@ -366,3 +366,5 @@ int spider_param_strict_group_by(
   THD *thd,
   int strict_group_by
 );
+bool spider_param_direct_aggregate(THD *thd);
+bool spider_param_disable_group_by_handler(THD *thd);
