@@ -344,7 +344,6 @@ bool spider_check_pk_update(
   TABLE *table
 );
 
-
 void spider_set_tmp_share_pointer(
   SPIDER_SHARE *tmp_share,
   char **tmp_connect_info,
@@ -446,7 +445,6 @@ double spider_rand(
   uint32 rand_source
 );
 
-#ifdef SPIDER_HAS_DISCOVER_TABLE_STRUCTURE
 int spider_discover_table_structure_internal(
   SPIDER_TRX *trx,
   SPIDER_SHARE *spider_share,
@@ -459,9 +457,7 @@ int spider_discover_table_structure(
   TABLE_SHARE *share,
   HA_CREATE_INFO *info
 );
-#endif
 
-#ifndef WITHOUT_SPIDER_BG_SEARCH
 int spider_create_spider_object_for_share(
   SPIDER_TRX *trx,
   SPIDER_SHARE *share,
@@ -472,12 +468,19 @@ void spider_free_spider_object_for_share(
   ha_spider **spider
 );
 
-int spider_create_sts_crd_threads(
-  SPIDER_THREAD *spider_thread,
-  bool is_sts
+int spider_create_sts_threads(
+  SPIDER_THREAD *spider_thread
 );
 
-void spider_free_sts_crd_threads(
+void spider_free_sts_threads(
+  SPIDER_THREAD *spider_thread
+);
+
+int spider_create_crd_threads(
+  SPIDER_THREAD *spider_thread
+);
+
+void spider_free_crd_threads(
   SPIDER_THREAD *spider_thread
 );
 
@@ -504,7 +507,6 @@ void spider_table_remove_share_from_sts_thread(
 void spider_table_remove_share_from_crd_thread(
   SPIDER_SHARE *share
 );
-#endif
 uchar *spider_duplicate_char(
   uchar *dst,
   uchar esc,
